# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import sys
from argparse import Namespace
from socket import timeout

from awscli.customizations.codedeploy.install import Install
from awscli.customizations.codedeploy.systems import (
    RHEL,
    System,
    Ubuntu,
    Windows,
)
from awscli.customizations.exceptions import (
    ConfigurationError,
    ParamValidationError,
)
from awscli.testutils import mock, unittest


class TestInstall(unittest.TestCase):
    def setUp(self):
        self.region = 'us-east-1'
        self.config_file = 'config-file'
        self.installer = 'install'
        self.bucket = f'aws-codedeploy-{self.region}'
        self.key = f'latest/{self.installer}'
        self.agent_installer = f's3://{self.bucket}/{self.key}'

        self.system_patcher = mock.patch('platform.system')
        self.system = self.system_patcher.start()
        self.system.return_value = 'Linux'

        self.linux_distribution_patcher = mock.patch(
            'awscli.compat.linux_distribution'
        )
        self.linux_distribution = self.linux_distribution_patcher.start()
        self.linux_distribution.return_value = ('Ubuntu', '', '')

        self.urlopen_patcher = mock.patch(
            'awscli.customizations.codedeploy.utils.urlopen'
        )
        self.urlopen = self.urlopen_patcher.start()
        self.urlopen.side_effect = timeout('Not EC2 instance')

        self.geteuid_patcher = mock.patch('os.geteuid', create=True)
        self.geteuid = self.geteuid_patcher.start()
        self.geteuid.return_value = 0

        self.isfile_patcher = mock.patch('os.path.isfile')
        self.isfile = self.isfile_patcher.start()
        self.isfile.return_value = False

        self.makedirs_patcher = mock.patch('os.makedirs')
        self.makedirs = self.makedirs_patcher.start()

        self.copyfile_patcher = mock.patch('shutil.copyfile')
        self.copyfile = self.copyfile_patcher.start()

        self.open_patcher = mock.patch(
            'awscli.customizations.codedeploy.systems.open',
            mock.mock_open(),
            create=True,
        )
        self.open = self.open_patcher.start()

        self.args = Namespace()
        self.args.override_config = False
        self.args.config_file = self.config_file
        self.args.agent_installer = None

        self.globals = Namespace()
        self.globals.region = self.region

        self.body = 'install-script'
        self.reader = mock.MagicMock()
        self.reader.read.return_value = self.body
        self.s3 = mock.MagicMock()
        self.s3.get_object.return_value = {'Body': self.reader}

        self.session = mock.MagicMock()
        self.session.create_client.return_value = self.s3
        self.install = Install(self.session)

    def tearDown(self):
        self.system_patcher.stop()
        self.linux_distribution_patcher.stop()
        self.urlopen_patcher.stop()
        self.geteuid_patcher.stop()
        self.isfile_patcher.stop()
        self.makedirs_patcher.stop()
        self.copyfile_patcher.stop()
        self.open_patcher.stop()

    def test_install_throws_on_invalid_region(self):
        self.globals.region = None
        self.session.get_config_variable.return_value = None
        error_msg = 'Region not specified.'
        with self.assertRaisesRegex(ConfigurationError, error_msg):
            self.install._run_main(self.args, self.globals)

    def test_install_throws_on_unsupported_system(self):
        self.system.return_value = 'Unsupported'
        with self.assertRaisesRegex(
            RuntimeError, System.UNSUPPORTED_SYSTEM_MSG
        ):
            self.install._run_main(self.args, self.globals)

    def test_install_throws_on_ec2_instance(self):
        self.urlopen.side_effect = None
        with self.assertRaisesRegex(
            RuntimeError, 'Amazon EC2 instances are not supported.'
        ):
            self.install._run_main(self.args, self.globals)
        self.assertIn('system', self.args)
        self.assertTrue(isinstance(self.args.system, Ubuntu))

    def test_install_throws_on_non_administrator(self):
        self.geteuid.return_value = 1
        with self.assertRaisesRegex(
            RuntimeError, 'You must run this command as sudo.'
        ):
            self.install._run_main(self.args, self.globals)

    def test_install_throws_on_no_override_config(self):
        self.isfile.return_value = True
        self.args.override_config = False
        with self.assertRaisesRegex(
            RuntimeError,
            'The on-premises instance configuration file already exists. '
            'Specify --override-config to update the existing on-premises '
            'instance configuration file.',
        ):
            self.install._run_main(self.args, self.globals)

    def test_install_throws_on_invalid_agent_installer(self):
        self.args.agent_installer = 'invalid-s3-location'
        with self.assertRaisesRegex(
            ParamValidationError,
            '--agent-installer must specify the Amazon S3 URL format as '
            's3://<bucket>/<key>.',
        ):
            self.install._run_main(self.args, self.globals)

    @mock.patch.object(Ubuntu, 'install')
    def test_install_with_agent_installer(self, install):
        self.args.agent_installer = self.agent_installer
        self.install._run_main(self.args, self.globals)
        self.assertIn('bucket', self.args)
        self.assertEqual(self.bucket, self.args.bucket)
        self.assertIn('key', self.args)
        self.assertEqual(self.key, self.args.key)
        self.assertIn('installer', self.args)
        self.assertEqual(self.installer, self.args.installer)
        install.assert_called_with(self.args)

    @mock.patch.object(Ubuntu, 'install')
    def test_install_for_ubuntu(self, install):
        self.system.return_value = 'Linux'
        self.linux_distribution.return_value = ('Ubuntu', '', '')
        self.install._run_main(self.args, self.globals)
        self.assertIn('bucket', self.args)
        self.assertEqual(self.bucket, self.args.bucket)
        self.assertIn('key', self.args)
        self.assertEqual('latest/install', self.args.key)
        self.assertIn('installer', self.args)
        self.assertEqual('install', self.args.installer)
        self.makedirs.assert_called_with('/etc/codedeploy-agent/conf')
        self.copyfile.assset_called_with(
            'codedeploy.onpremises.yml',
            '/etc/codedeploy-agent/conf/codedeploy.onpremises.yml',
        )
        install.assert_called_with(self.args)

    @mock.patch.object(Windows, 'install')
    @mock.patch.object(Windows, 'validate_administrator')
    def test_install_for_windows(self, validate_administrator, install):
        self.system.return_value = 'Windows'
        self.install._run_main(self.args, self.globals)
        self.assertIn('bucket', self.args)
        self.assertEqual(self.bucket, self.args.bucket)
        self.assertIn('key', self.args)
        self.assertEqual('latest/codedeploy-agent.msi', self.args.key)
        self.assertIn('installer', self.args)
        self.assertEqual('codedeploy-agent.msi', self.args.installer)
        self.makedirs.assert_called_with(r'C:\ProgramData\Amazon\CodeDeploy')
        self.copyfile.assset_called_with(
            'conf.onpremises.yml',
            r'C:\ProgramData\Amazon\CodeDeploy\conf.onpremises.yml',
        )
        validate_administrator.assert_called_with()
        install.assert_called_with(self.args)


if __name__ == "__main__":
    unittest.main()



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import textwrap

from awscli.autoprompt.doc import DocsGetter
from awscli.clidriver import create_clidriver
from awscli.testutils import mock, unittest


class TestDocsGetter(unittest.TestCase):
    def setUp(self):
        self.driver = create_clidriver()
        self.help_command = self.driver.create_help_command()
        self.help_command.doc = mock.Mock()
        self.help_command.doc.getvalue = mock.Mock()
        self.base_docs_getter = DocsGetter(self.driver)

    def test_get_doc_content(self):
        self.help_command.doc.getvalue.return_value = b'Dummy content.'
        content = self.base_docs_getter.get_doc_content(self.help_command)
        self.assertEqual(content, 'Dummy content.\n')

    def test_get_rst_doc_in_txt(self):
        content = textwrap.dedent("""\
            MySection
            =========

            This is some text.
            Here's a list:

            * foo
            * bar

            Literal text: ``--foo-bar``
        """)
        expected_response = textwrap.dedent("""\

            MYSECTION

            This is some text. Here's a list:

            * foo

            * bar

            Literal text: --foo-bar
        """)
        self.help_command.doc.getvalue.return_value = content.encode('utf-8')
        response = self.base_docs_getter.get_doc_content(self.help_command)
        self.assertEqual(response, expected_response)

    def test_get_doc_content_strips_carriage_returns(self):
        content = textwrap.dedent("""\
            MySection\r
            =========\r
        """)
        expected_response = textwrap.dedent("""\

            MYSECTION
        """)
        self.help_command.doc.getvalue.return_value = content.encode('utf-8')
        response = self.base_docs_getter.get_doc_content(self.help_command)
        self.assertEqual(response, expected_response)

    def test_get_empty_doc_content(self):
        self.help_command.doc.getvalue.return_value = b''
        content = self.base_docs_getter.get_doc_content(self.help_command)
        self.assertEqual(content, '')



from awscli.customizations.codeartifact import (
    inject_commands,
    register_codeartifact_commands,
)
from awscli.customizations.codeartifact.login import CodeArtifactLogin
from awscli.testutils import mock, unittest


class TestRegisterCodeArtifactCommands(unittest.TestCase):
    def test_register_codeartifact_commands(self):
        event_emitter = mock.Mock()
        register_codeartifact_commands(event_emitter)
        event_emitter.register.assert_called_once_with(
            'building-command-table.codeartifact', inject_commands
        )


class TestInjectCommands(unittest.TestCase):
    def test_inject_commands(self):
        command_table = {}
        session = mock.Mock()
        inject_commands(command_table, session)
        self.assertIn('login', command_table)
        self.assertIsInstance(command_table['login'], CodeArtifactLogin)



# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import datetime
import unittest

from awscli.customizations.s3.comparator import Comparator
from awscli.customizations.s3.filegenerator import FileStat
from awscli.testutils import mock


class ComparatorTest(unittest.TestCase):
    def setUp(self):
        self.sync_strategy = mock.Mock()
        self.not_at_src_sync_strategy = mock.Mock()
        self.not_at_dest_sync_strategy = mock.Mock()
        self.comparator = Comparator(
            self.sync_strategy,
            self.not_at_dest_sync_strategy,
            self.not_at_src_sync_strategy,
        )

    def test_compare_key_equal_should_not_sync(self):
        """
        Confirm the appropriate action is taken when the soruce compare key
        is equal to the destination compare key.
        """
        # Try when the sync strategy says not to sync the file.
        self.sync_strategy.determine_should_sync.return_value = False

        src_files = []
        dest_files = []
        ref_list = []
        result_list = []
        time = datetime.datetime.now()
        src_file = FileStat(
            src='',
            dest='',
            compare_key='comparator_test.py',
            size=10,
            last_update=time,
            src_type='local',
            dest_type='s3',
            operation_name='upload',
        )
        dest_file = FileStat(
            src='',
            dest='',
            compare_key='comparator_test.py',
            size=10,
            last_update=time,
            src_type='s3',
            dest_type='local',
            operation_name='',
        )
        src_files.append(src_file)
        dest_files.append(dest_file)
        files = self.comparator.call(iter(src_files), iter(dest_files))
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)

        # Try when the sync strategy says to sync the file.
        self.sync_strategy.determine_should_sync.return_value = True

        ref_list = []
        result_list = []
        files = self.comparator.call(iter(src_files), iter(dest_files))
        ref_list.append(src_file)
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)

    def test_compare_key_less(self):
        """
        Confirm the appropriate action is taken when the soruce compare key
        is less than the destination compare key.
        """
        self.not_at_src_sync_strategy.determine_should_sync.return_value = (
            False
        )

        # Try when the sync strategy says to sync the file.
        self.not_at_dest_sync_strategy.determine_should_sync.return_value = (
            True
        )

        src_files = []
        dest_files = []
        ref_list = []
        result_list = []
        time = datetime.datetime.now()
        src_file = FileStat(
            src='',
            dest='',
            compare_key='bomparator_test.py',
            size=10,
            last_update=time,
            src_type='local',
            dest_type='s3',
            operation_name='upload',
        )
        dest_file = FileStat(
            src='',
            dest='',
            compare_key='comparator_test.py',
            size=10,
            last_update=time,
            src_type='s3',
            dest_type='local',
            operation_name='',
        )
        src_files.append(src_file)
        dest_files.append(dest_file)
        ref_list.append(src_file)
        files = self.comparator.call(iter(src_files), iter(dest_files))
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)

        # Now try when the sync strategy says not to sync the file.
        self.not_at_dest_sync_strategy.determine_should_sync.return_value = (
            False
        )
        result_list = []
        ref_list = []
        files = self.comparator.call(iter(src_files), iter(dest_files))
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)

    def test_compare_key_greater(self):
        """
        Confirm the appropriate action is taken when the soruce compare key
        is greater than the destination compare key.
        """
        self.not_at_dest_sync_strategy.determine_should_sync.return_value = (
            False
        )

        # Try when the sync strategy says to sync the file.
        self.not_at_src_sync_strategy.determine_should_sync.return_value = True

        src_files = []
        dest_files = []
        ref_list = []
        result_list = []
        time = datetime.datetime.now()
        src_file = FileStat(
            src='',
            dest='',
            compare_key='domparator_test.py',
            size=10,
            last_update=time,
            src_type='local',
            dest_type='s3',
            operation_name='upload',
        )
        dest_file = FileStat(
            src='',
            dest='',
            compare_key='comparator_test.py',
            size=10,
            last_update=time,
            src_type='s3',
            dest_type='local',
            operation_name='',
        )
        src_files.append(src_file)
        dest_files.append(dest_file)
        ref_list.append(dest_file)
        files = self.comparator.call(iter(src_files), iter(dest_files))
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)

        # Now try when the sync strategy says not to sync the file.
        self.not_at_src_sync_strategy.determine_should_sync.return_value = (
            False
        )
        result_list = []
        ref_list = []
        files = self.comparator.call(iter(src_files), iter(dest_files))
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)

    def test_empty_src(self):
        """
        Confirm the appropriate action is taken when there are no more source
        files to take.
        """
        # Try when the sync strategy says to sync the file.
        self.not_at_src_sync_strategy.determine_should_sync.return_value = True

        src_files = []
        dest_files = []
        ref_list = []
        result_list = []
        time = datetime.datetime.now()
        dest_file = FileStat(
            src='',
            dest='',
            compare_key='comparator_test.py',
            size=10,
            last_update=time,
            src_type='s3',
            dest_type='local',
            operation_name='',
        )
        dest_files.append(dest_file)
        ref_list.append(dest_file)
        files = self.comparator.call(iter(src_files), iter(dest_files))
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)

        # Now try when the sync strategy says not to sync the file.
        self.not_at_src_sync_strategy.determine_should_sync.return_value = (
            False
        )
        result_list = []
        ref_list = []
        files = self.comparator.call(iter(src_files), iter(dest_files))
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)

    def test_empty_dest(self):
        """
        Confirm the appropriate action is taken when there are no more dest
        files to take.
        """
        # Try when the sync strategy says to sync the file.
        self.not_at_dest_sync_strategy.determine_should_sync.return_value = (
            True
        )

        src_files = []
        dest_files = []
        ref_list = []
        result_list = []
        time = datetime.datetime.now()
        src_file = FileStat(
            src='',
            dest='',
            compare_key='domparator_test.py',
            size=10,
            last_update=time,
            src_type='local',
            dest_type='s3',
            operation_name='upload',
        )
        src_files.append(src_file)
        ref_list.append(src_file)
        files = self.comparator.call(iter(src_files), iter(dest_files))
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)

        # Now try when the sync strategy says not to sync the file.
        self.not_at_dest_sync_strategy.determine_should_sync.return_value = (
            False
        )
        result_list = []
        ref_list = []
        files = self.comparator.call(iter(src_files), iter(dest_files))
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)

    def test_empty_src_dest(self):
        """
        Confirm the appropriate action is taken when there are no more
        files to take for both source and destination.
        """
        src_files = []
        dest_files = []
        ref_list = []
        result_list = []
        files = self.comparator.call(iter(src_files), iter(dest_files))
        for filename in files:
            result_list.append(filename)
        self.assertEqual(result_list, ref_list)


if __name__ == "__main__":
    unittest.main()



from awscli.customizations.emr.createcluster import CreateCluster
from awscli.customizations.emr.exceptions import InvalidBooleanConfigError
from awscli.customizations.emr.ssh import SSH, Get, Put, Socks
from awscli.testutils import BaseAWSHelpOutputTest, mock
from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)

INSTANCE_GROUPS_ARG = (
    'InstanceGroupType=MASTER,Name=MASTER,'
    'InstanceCount=1,InstanceType=m1.large '
)

CREATE_CLUSTER_CMD = (
    "emr create-cluster --ami-version 3.1.0 --instance-groups %s"
    % INSTANCE_GROUPS_ARG
)

DEFAULT_CONFIGS = {
    'service_role': 'my_default_emr_role',
    'instance_profile': 'my_default_ec2_role',
    'log_uri': 's3://my_default_logs',
    'enable_debugging': 'True',
    'key_name': 'my_default_key',
    'key_pair_file': '/home/my_default_key_pair.pem',
}

DEBUG_FALSE_CONFIGS = {'enable_debugging': 'False'}

BAD_BOOLEAN_VALUE_CONFIGS = {'enable_debugging': 'False1'}

BAD_CONFIGS = {
    'service_role1': 'my_default_emr_role',
    'instance_profiile': 'my_default_ec2_role',
    'log_uri_': 's3://my_default_logs',
    'enable_debugging*': 'True',
    'keyname': 'my_default_key',
    'key_pairrr_file': '/home/my_default_key_pair.pem',
}

TEST_CLUSTER_ID = "j-227H3PFKLBOBP"
TEST_SRC_PATH = "/home/my_src"

SSH_CMD = 'emr ssh --cluster-id %s' % TEST_CLUSTER_ID
SOCKS_CMD = 'emr socks --cluster-id %s' % TEST_CLUSTER_ID
GET_CMD = 'emr get --cluster-id %s --src %s' % (TEST_CLUSTER_ID, TEST_SRC_PATH)
PUT_CMD = 'emr put --cluster-id %s --src %s' % (TEST_CLUSTER_ID, TEST_SRC_PATH)

CREATE_DEFAULT_ROLES_CMD = 'emr create-default-roles'


class TestCreateCluster(BaseAWSCommandParamsTest):
    @mock.patch.object(CreateCluster, '_run_main_command')
    def test_with_configs(self, mock_run_main_command):
        self.set_configs(DEFAULT_CONFIGS)
        self._run_cmd_w_mock(mock_run_main_command, CREATE_CLUSTER_CMD)
        parsed_args = mock_run_main_command.call_args[0][0]
        self._assert_args_for_default_configs(parsed_args)

    @mock.patch.object(CreateCluster, '_run_main_command')
    def test_with_configs_and_other_ec2_attributes(
        self, mock_run_main_command
    ):
        self.set_configs(DEFAULT_CONFIGS)
        cmd = (
            CREATE_CLUSTER_CMD
            + ' --ec2-attributes AvailabilityZone=us-east-1e'
        )
        self._run_cmd_w_mock(mock_run_main_command, cmd)
        parsed_args = mock_run_main_command.call_args[0][0]
        self._assert_args_for_default_configs(parsed_args)

    @mock.patch.object(CreateCluster, '_run_main_command')
    def test_enable_debugging_false(self, mock_run_main_command):
        self.set_configs(DEBUG_FALSE_CONFIGS)
        self._run_cmd_w_mock(mock_run_main_command, CREATE_CLUSTER_CMD)
        parsed_args = mock_run_main_command.call_args[0][0]
        self._assert_enable_debugging(parsed_args, False)

    @mock.patch.object(CreateCluster, '_run_main_command')
    def test_with_bad_configs(self, mock_run_main_command):
        self.set_configs(BAD_CONFIGS)
        self._run_cmd_w_mock(mock_run_main_command, CREATE_CLUSTER_CMD)
        parsed_args = mock_run_main_command.call_args[0][0]
        self.assertFalse(getattr(parsed_args, 'service_role', None))
        self.assertFalse(getattr(parsed_args, 'log_uri', None))
        self.assertFalse(parsed_args.enable_debugging)
        self.assertFalse(parsed_args.no_enable_debugging)
        self.assertFalse(getattr(parsed_args, 'ec2-attributes', None))

    def test_with_bad_boolean_value(self):
        self.set_configs(BAD_BOOLEAN_VALUE_CONFIGS)
        cmd = CREATE_CLUSTER_CMD
        expect_error_msg = (
            "\naws: [ERROR]: An error occurred (ParamValidation): %s\n"
            % InvalidBooleanConfigError.fmt.format(
                config_value='False1',
                config_key='enable_debugging',
                profile_var_name='default',
            )
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expect_error_msg, result[1])

    @mock.patch.object(CreateCluster, '_run_main_command')
    def test_ignore_role_configs_when_use_default_roles(
        self, mock_run_main_command
    ):
        self.set_configs(DEFAULT_CONFIGS)
        cmd = CREATE_CLUSTER_CMD + '--use-default-roles'
        self._run_cmd_w_mock(mock_run_main_command, cmd)
        parsed_args = mock_run_main_command.call_args[0][0]
        self._assert_enable_debugging(parsed_args, True)
        self._assert_log_uri(parsed_args, 's3://my_default_logs')
        self._assert_ec2_attributes(parsed_args, 'my_default_key')

    @mock.patch.object(CreateCluster, '_run_main_command')
    def test_with_configs_and_overriding_options(self, mock_run_main_command):
        self.set_configs(DEFAULT_CONFIGS)
        cmd = (
            CREATE_CLUSTER_CMD
            + ' --service-role my_emr_role --log-uri s3://my_logs'
            ' --ec2-attributes KeyName=my_key,'
            'InstanceProfile=my_ec2_role --no-enable-debugging'
        )
        self._run_cmd_w_mock(mock_run_main_command, cmd)
        parsed_args = mock_run_main_command.call_args[0][0]
        self._assert_enable_debugging(parsed_args, False)
        self._assert_log_uri(parsed_args, 's3://my_logs')
        self._assert_service_role(parsed_args, 'my_emr_role')
        self._assert_ec2_attributes(parsed_args, 'my_key', 'my_ec2_role')

    @mock.patch.object(CreateCluster, '_run_main_command')
    def test_with_configs_and_overriding_ec2_attributes(
        self, mock_run_main_command
    ):
        self.set_configs(DEFAULT_CONFIGS)
        cmd = (
            CREATE_CLUSTER_CMD
            + ' --ec2-attributes AvailabilityZone=us-east-1e,'
            'KeyName=my_key'
        )
        self._run_cmd_w_mock(mock_run_main_command, cmd)
        parsed_args = mock_run_main_command.call_args[0][0]
        self._assert_enable_debugging(parsed_args, True)
        self._assert_log_uri(parsed_args, 's3://my_default_logs')
        self._assert_service_role(parsed_args, 'my_default_emr_role')
        self._assert_ec2_attributes(
            parsed_args, 'my_key', 'my_default_ec2_role'
        )

    def _run_cmd_w_mock(self, mock_run_main_command, cmd, return_value=0):
        mock_run_main_command.return_value = return_value
        self.run_cmd(cmd, return_value)

    def _assert_args_for_default_configs(self, parsed_args):
        self._assert_enable_debugging(parsed_args, True)
        self._assert_log_uri(parsed_args, 's3://my_default_logs')
        self._assert_service_role(parsed_args, 'my_default_emr_role')
        self._assert_ec2_attributes(
            parsed_args, 'my_default_key', 'my_default_ec2_role'
        )

    def _assert_enable_debugging(self, parsed_args, value):
        if value:
            self.assertTrue(parsed_args.enable_debugging)
            self.assertFalse(parsed_args.no_enable_debugging)
        else:
            self.assertFalse(parsed_args.enable_debugging)
            self.assertTrue(parsed_args.no_enable_debugging)

    def _assert_ec2_attributes(
        self, parsed_args, key_name=None, instance_profile=None
    ):
        if key_name:
            self.assertEqual(parsed_args.ec2_attributes['KeyName'], key_name)
        if instance_profile:
            self.assertEqual(
                parsed_args.ec2_attributes['InstanceProfile'], instance_profile
            )

    def _assert_log_uri(self, parsed_args, value):
        self.assertEqual(parsed_args.log_uri, value)

    def _assert_service_role(self, parsed_args, value):
        self.assertEqual(parsed_args.service_role, value)


class TestSSHBasedCommands(BaseAWSCommandParamsTest):
    @mock.patch.object(SSH, '_run_main_command')
    def test_ssh_with_configs(self, mock_run_main_command):
        self._test_cmd_with_configs(mock_run_main_command, SSH_CMD)

    @mock.patch.object(Socks, '_run_main_command')
    def test_socks_with_configs(self, mock_run_main_command):
        self._test_cmd_with_configs(mock_run_main_command, SOCKS_CMD)

    @mock.patch.object(Get, '_run_main_command')
    def test_get_with_configs(self, mock_run_main_command):
        self._test_cmd_with_configs(mock_run_main_command, GET_CMD)

    @mock.patch.object(Put, '_run_main_command')
    def test_put_with_configs(self, mock_run_main_command):
        self._test_cmd_with_configs(mock_run_main_command, PUT_CMD)

    @mock.patch.object(SSH, '_run_main_command')
    def test_ssh_with_configs_and_overriding_key(self, mock_run_main_command):
        cmd = SSH_CMD + " --key-pair-file /home/my_key_pair.pem"
        self._test_cmd_with_configs(
            mock_run_main_command, cmd, '/home/my_key_pair.pem'
        )

    def _test_cmd_with_configs(
        self,
        mock_run_main_command,
        cmd,
        key_pair_file_to_assert='' '/home/my_default_key_pair.pem',
    ):
        self.set_configs(DEFAULT_CONFIGS)
        mock_run_main_command.return_value = 0
        self.run_cmd(cmd, 0)
        call_args = mock_run_main_command.call_args
        self.assertEqual(
            call_args[0][0].key_pair_file, key_pair_file_to_assert
        )


class TestHelpOutput(BaseAWSHelpOutputTest):
    def test_not_override_required_options(self):
        scoped_config = self.session.get_scoped_config()
        scoped_config['emr'] = DEFAULT_CONFIGS
        self.driver.main(['emr', 'ssh', 'help'])
        self.assert_contains('--key-pair-file <value>')
        self.assert_not_contains('[--key-pair-file <value>]')


class TestCreateDefaultRoles(BaseAWSCommandParamsTest):
    def test_roles_updated_if_not_present(self):
        self.run_cmd(CREATE_DEFAULT_ROLES_CMD, expected_rc=0)

        call_args_list = self.mock_update_config.call_args_list

        keys = [x[0][0] for x in call_args_list]
        self.assertTrue('instance_profile' in keys)
        self.assertTrue('service_role' in keys)

    def test_roles_not_updated_if_present(self):
        self.set_configs(DEFAULT_CONFIGS)
        self.run_cmd(CREATE_DEFAULT_ROLES_CMD, expected_rc=0)

        call_args_list = self.mock_update_config.call_args_list

        self.assertFalse(call_args_list)


if __name__ == "__main__":
    unittest.main()



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import copy
import unittest

import ruamel.yaml as yaml

unittest.TestCase.assertItemsEqual = unittest.TestCase.assertCountEqual


class BaseYAMLTest(unittest.TestCase):
    """Base class for preserving ruamel defaults

    Because ruamel mutates module and class state when custom components
    are added to a loader or a dumper, it will force downstream tests to pick
    up these customizations and cause unexpected failures. This class is meant
    to preserve any ruamel defaults to prevent downstream test failures. Note
    that mutating module and class state is not really a problem though when a
    CLI command runs because we do not run multiple commands that differ
    in YAML loading logic in a single process.
    """

    def setUp(self):
        super(BaseYAMLTest, self).setUp()
        self.original_implicit_resolvers = copy.deepcopy(
            yaml.resolver.implicit_resolvers
        )

    def tearDown(self):
        super(BaseYAMLTest, self).tearDown()
        yaml.resolver.implicit_resolvers = self.original_implicit_resolvers



#!/usr/bin/env python
# Copyright 2012-2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import json
import locale
import os
import re
import sys

from awscli.compat import StringIO
from awscli.formatter import Formatter
from awscli.testutils import BaseAWSCommandParamsTest, mock, unittest


class TestListUsers(BaseAWSCommandParamsTest):
    def setUp(self):
        super(TestListUsers, self).setUp()
        self.first_parsed_response = {
            'Users': [
                {
                    "UserName": "testuser-50",
                    "Path": "/",
                    "CreateDate": "2013-02-12T19:08:52Z",
                    "UserId": "EXAMPLEUSERID",
                    "Arn": "arn:aws:iam::12345:user/testuser1",
                },
            ],
            'Groups': [],
        }
        self.second_parsed_response = {
            'Users': [
                {
                    "UserName": "testuser-51",
                    "Path": "/",
                    "CreateDate": "2012-10-14T23:53:39Z",
                    "UserId": "EXAMPLEUSERID",
                    "Arn": "arn:aws:iam::123456:user/testuser2",
                },
            ],
            'Groups': [],
        }

        self.parsed_responses = [
            self.first_parsed_response,
            self.second_parsed_response,
        ]

    def test_text_response(self):
        output = self.run_cmd('iam list-users --output text', expected_rc=0)[0]
        self.assertEqual(
            output,
            (
                'USERS\tarn:aws:iam::12345:user/testuser1\t2013-02-12T19:08:52Z\t'
                '/\tEXAMPLEUSERID\ttestuser-50\n'
            ),
        )

        # Test something with a jmespath expression.
        output = self.run_cmd(
            'rds describe-engine-default-parameters '
            '--db-parameter-group-family mysql5.1 --output text',
            expected_rc=0,
        )[0]
        self.assertEqual(output, 'ENGINEDEFAULTS\tNone\n')


class TestDescribeChangesets(BaseAWSCommandParamsTest):
    def setUp(self):
        super(TestDescribeChangesets, self).setUp()
        self.first_parsed_response = {
            'Capabilities': ['CAPABILITY_IAM'],
            'ChangeSetId': (
                'arn:aws:cloudformation:us-west-2:12345:changeSet'
                '/mychangeset/12345'
            ),
            'ChangeSetName': 'mychangeset',
            'Changes': [{"ChangeId": "1"}],
            'CreationTime': '2019-04-08T14:21:53.765Z',
            'ExecutionStatus': 'AVAILABLE',
            'NotificationARNs': [],
            'RollbackConfiguration': {'RollbackTriggers': []},
            'StackId': (
                'arn:aws:cloudformation:us-west-2:12345:stack' '/MyStack/12345'
            ),
            'StackName': 'MyStack',
            'Status': 'CREATE_COMPLETE',
            'NextToken': "more stuff",
        }
        self.second_parsed_response = {
            'Capabilities': ['CAPABILITY_IAM'],
            'ChangeSetId': (
                'arn:aws:cloudformation:us-west-2:12345:changeSet'
                '/mychangeset/12345'
            ),
            'ChangeSetName': 'mychangeset',
            'Changes': [{"ChangeId": "2"}],
            'CreationTime': '2019-04-08T14:21:53.765Z',
            'ExecutionStatus': 'AVAILABLE',
            'NotificationARNs': [],
            'RollbackConfiguration': {'RollbackTriggers': []},
            'StackId': (
                'arn:aws:cloudformation:us-west-2:12345:stack' '/MyStack/12345'
            ),
            'StackName': 'MyStack',
            'Status': 'CREATE_COMPLETE',
        }
        self.parsed_responses = [
            self.first_parsed_response,
            self.second_parsed_response,
        ]

    def test_non_aggregate_keys(self):
        output = self.run_cmd(
            (
                'cloudformation describe-change-set --change-set-name mychangeset'
                ' --stack-name MyStack --output text'
            ),
            expected_rc=0,
        )[0]
        fields = output.split()
        self.assertIn(
            (
                "arn:aws:cloudformation:us-west-2:12345:changeSet/mychangeset/"
                "12345"
            ),
            fields,
        )
        self.assert_in("CAPABILITY_IAM", fields, 1)
        self.assert_in("mychangeset", fields, 1)
        self.assert_in("2019-04-08T14:21:53.765Z", fields, 1)
        self.assert_in("AVAILABLE", fields, 1)
        self.assert_in("MyStack", fields, 1)
        self.assert_in("CREATE_COMPLETE", fields, 1)

    def assert_in(self, key, fields, count=None):
        if count is None:
            self.assertIn(key, fields)
        else:
            actual_count = fields.count(key)
            self.assertEqual(
                count,
                actual_count,
                "%s was found in the output %s times. Expected %s."
                % (key, actual_count, count),
            )



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import random

import botocore.session
import pytest

from awscli.testutils import aws, random_chars, unittest


class TestDynamoDBWait(unittest.TestCase):
    def setUp(self):
        self.session = botocore.session.get_session()
        self.client = self.session.create_client('dynamodb', 'us-west-2')

    @pytest.mark.slow
    def test_wait_table_exists(self):
        # Create a table.
        table_name = 'awscliddb-%s' % random_chars(10)
        self.client.create_table(
            TableName=table_name,
            ProvisionedThroughput={
                "ReadCapacityUnits": 5,
                "WriteCapacityUnits": 5,
            },
            KeySchema=[{"AttributeName": "foo", "KeyType": "HASH"}],
            AttributeDefinitions=[
                {"AttributeName": "foo", "AttributeType": "S"}
            ],
        )
        self.addCleanup(self.client.delete_table, TableName=table_name)

        # Wait for the table to be active.
        p = aws(
            'dynamodb wait table-exists --table-name %s --region us-west-2'
            % table_name
        )
        self.assertEqual(p.rc, 0)

        # Make sure the table is active.
        parsed = self.client.describe_table(TableName=table_name)
        self.assertEqual(parsed['Table']['TableStatus'], 'ACTIVE')



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
import re
import shutil
import subprocess
import sys
import venv
from pathlib import Path

from constants import DIST_INFO_DIRECTORIES_TO_KEEP

IS_WINDOWS = sys.platform == "win32"
BIN_DIRNAME = "Scripts" if IS_WINDOWS else "bin"
PYTHON_EXE_NAME = "python.exe" if IS_WINDOWS else "python"
CLI_SCRIPT_NAME = "aws.cmd" if IS_WINDOWS else "aws"
LOCK_SUFFIX = "win-lock.txt" if IS_WINDOWS else "lock.txt"

ROOT = Path(__file__).parents[4]
BOOTSTRAP_REQUIREMENTS = (
    ROOT / "requirements" / "download-deps" / f"bootstrap-{LOCK_SUFFIX}"
)
PORTABLE_EXE_REQUIREMENTS = (
    ROOT / "requirements" / "download-deps" / f"portable-exe-{LOCK_SUFFIX}"
)
SYSTEM_SANDBOX_REQIREMENTS = (
    ROOT / "requirements" / "download-deps" / f"system-sandbox-{LOCK_SUFFIX}"
)


class BaseArtifactTest:
    def expected_cli_version(self):
        init_file_path = ROOT / "awscli" / "__init__.py"
        version_file = open(init_file_path).read()
        version_match = re.search(
            r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M
        )
        if version_match:
            return version_match.group(1)
        raise RuntimeError("Unable to find version string.")

    def expected_python_version(self):
        python_info = sys.version_info
        return f"{python_info.major}.{python_info.minor}.{python_info.micro}"

    def assert_version_string_is_correct(self, exe_path, dist_type):
        version_string = subprocess.check_output(
            [str(exe_path), "--version"]
        ).decode()

        assert f"aws-cli/{self.expected_cli_version()}" in version_string
        assert f"Python/{self.expected_python_version()}" in version_string
        assert f"source-{dist_type}" in version_string

    def assert_no_pycache(self, directory: Path):
        failures = []
        for root, dirs, files in os.walk(directory):
            # Do not check the build directory since that is expected to have
            # __pycache__ folders generated inside it.
            if 'build' in dirs:
                dirs.remove('build')
            # If we are not in a build directory there should not be
            # any pycache directories.
            if '__pycache__' in dirs:
                failures.append(os.path.join(root, '__pycache__'))

        assert (
            failures == []
        ), f"Expected no __pycache__ directories, found {failures}"

    def assert_built_venv_is_correct(self, venv_dir):
        self.assert_venv_is_correct(venv_dir)
        files = os.listdir(venv_dir)

        # The venv after building also includes the binary
        assert BIN_DIRNAME in files
        aws_exe = venv_dir / BIN_DIRNAME / CLI_SCRIPT_NAME
        self.assert_version_string_is_correct(aws_exe, "sandbox")

    def assert_venv_is_correct(self, venv_dir):
        files = os.listdir(venv_dir)
        assert "Include" in files or "include" in files
        assert "Lib" in files or "lib" in files or "lib64" in files
        assert "pyvenv.cfg" in files

    def assert_built_exe_is_correct(self, root_dir):
        aws_dir = root_dir / "build" / "exe" / "aws"
        files = os.listdir(aws_dir)

        assert set(files) == {
            "README.md",
            "THIRD_PARTY_LICENSES",
            "dist",
            "install",
        }
        dist_dir = aws_dir / "dist"
        dist_info_files = set(
            f
            for f in os.listdir(dist_dir)
            if '.dist-info' in f
            and not any(keep in f for keep in DIST_INFO_DIRECTORIES_TO_KEEP)
        )
        assert (
            dist_info_files == set()
        ), f"Expected no dist-info files (except {DIST_INFO_DIRECTORIES_TO_KEEP}), found: {dist_info_files}"

        aws_exe = aws_dir / "dist" / "aws"
        self.assert_version_string_is_correct(aws_exe, "exe")

    def assert_installed_exe_is_correct(self, exe_dir):
        self.assert_version_string_is_correct(exe_dir / CLI_SCRIPT_NAME, "exe")

    def assert_installed_venv_is_correct(self, exe_dir, lib_dir):
        self.assert_version_string_is_correct(
            exe_dir / CLI_SCRIPT_NAME, "sandbox"
        )
        self.assert_venv_is_correct(lib_dir / "aws-cli")


class VEnvWorkspace:
    def __init__(self, path):
        self.path = path
        self.cli_path = path / "aws"
        self.venv_path = path / "venv"
        self.install_path = path / "install"
        self.bin_path = path / "install" / "bin"
        self.lib_path = path / "install" / "lib"

        self._init_cli_directory()
        self._init_venv_directory()
        self._init_install_path()

    def _init_cli_directory(self):
        shutil.copytree(
            ROOT,
            self.cli_path,
            ignore=shutil.ignore_patterns(
                ".git",
                "build",
                ".tox",
                "__pycache__",
            ),
        )

    def _init_venv_directory(self):
        venv.create(self.venv_path, with_pip=True)

    def _init_install_path(self):
        self.install_path.mkdir()
        self.bin_path.mkdir()
        self.lib_path.mkdir()

    def install_bootstrap_dependencies(self):
        self._install_requirements_file(BOOTSTRAP_REQUIREMENTS)

    def install_dependencies(self):
        self.install_bootstrap_dependencies()
        self._install_requirements_file(SYSTEM_SANDBOX_REQIREMENTS)

    def install_pyinstaller(self):
        self._install_requirements_file(PORTABLE_EXE_REQUIREMENTS)

    def _install_requirements_file(self, path: Path):
        subprocess.check_call(
            [self.python_exe(), "-m", "pip", "install", "-r", path]
        )

    @property
    def build_path(self):
        return self.cli_path / "build" / "venv"

    def python_exe(self):
        return self.venv_path / BIN_DIRNAME / PYTHON_EXE_NAME

    def env(self, overrides: dict[str, str] = None):
        env = os.environ.copy()
        if overrides:
            env.update(overrides)
        env["PYTHON"] = str(self.python_exe())
        return env

    def subprocess(self, args, env=None):
        return subprocess.check_output(
            args,
            stderr=subprocess.STDOUT,
            cwd=self.cli_path,
            env=self.env(env),
        )

    def configure(self, install_type: str, download_deps: bool = False):
        configure_path = self.cli_path / "configure"
        args = [
            configure_path,
            f"--with-install-type={install_type}",
        ]
        if download_deps:
            args.append("--with-download-deps")
        self.subprocess(args)

    def make(self, args=None, env=None):
        cmd = ["make"]
        if args:
            cmd += args
        return self.subprocess(cmd, env=env)

    def call_build_system(self, artifact_type: str, download_deps: bool):
        args = [
            self.python_exe(),
            os.path.join("backends", "build_system"),
            "build",
            "--artifact",
            artifact_type,
            "--build-dir",
            "build",
        ]
        if download_deps:
            args.append("--download-deps")
        return self.subprocess(args)

    def call_install(self, bin_path: str, lib_path: str):
        args = [
            self.python_exe(),
            os.path.join("backends", "build_system"),
            "install",
            "--bin-dir",
            bin_path,
            "--lib-dir",
            lib_path,
            "--build-dir",
            "build",
        ]
        return self.subprocess(args)

    def call_uninstall(self, bin_path: str, lib_path: str):
        args = [
            self.python_exe(),
            os.path.join("backends", "build_system"),
            "uninstall",
            "--bin-dir",
            bin_path,
            "--lib-dir",
            lib_path,
        ]
        return self.subprocess(args)



# Copyright 2016 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestRb(BaseAWSCommandParamsTest):
    prefix = 's3 rb '

    def test_rb(self):
        command = self.prefix + 's3://bucket'
        self.run_cmd(command)
        self.assertEqual(len(self.operations_called), 1)
        self.assertEqual(self.operations_called[0][0].name, 'DeleteBucket')

    def test_rb_force_empty_bucket(self):
        command = self.prefix + 's3://bucket --force'
        self.run_cmd(command)
        self.assertEqual(len(self.operations_called), 2)
        self.assertEqual(self.operations_called[0][0].name, 'ListObjectsV2')
        self.assertEqual(self.operations_called[1][0].name, 'DeleteBucket')

    def test_rb_force_non_empty_bucket(self):
        command = self.prefix + 's3://bucket --force'
        self.parsed_responses = [
            {
                'Contents': [
                    {
                        'Key': 'foo',
                        'Size': 100,
                        'LastModified': '2016-03-01T23:50:13.000Z',
                    }
                ]
            },
            {},
            {},
        ]
        self.run_cmd(command)
        self.assertEqual(len(self.operations_called), 3)
        self.assertEqual(self.operations_called[0][0].name, 'ListObjectsV2')
        self.assertEqual(self.operations_called[1][0].name, 'DeleteObject')
        self.assertEqual(self.operations_called[2][0].name, 'DeleteBucket')

    def test_rb_failed_rc(self):
        command = self.prefix + 's3://bucket'
        self.http_response.status_code = 500
        _, stderr, _ = self.run_cmd(command, expected_rc=1)
        self.assertIn('remove_bucket failed:', stderr)

    def test_rb_force_with_failed_rm(self):
        command = self.prefix + 's3://bucket --force'
        self.http_response.status_code = 500
        _, stderr, _ = self.run_cmd(command, expected_rc=255)
        self.assertIn('remove_bucket failed:', stderr)
        self.assertEqual(len(self.operations_called), 1)
        self.assertEqual(self.operations_called[0][0].name, 'ListObjectsV2')

    def test_nonzero_exit_if_uri_scheme_not_provided(self):
        command = self.prefix + 'bucket'
        self.run_cmd(command, expected_rc=252)

    def test_nonzero_exit_if_key_provided(self):
        command = self.prefix + 's3://bucket/key --force'
        self.run_cmd(command, expected_rc=252)

        command = self.prefix + 's3://bucket/key'
        self.run_cmd(command, expected_rc=252)



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from copy import deepcopy

from awscli.testutils import mock
from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)


class TestRestoreFromHBaseBackup(BaseAWSCommandParamsTest):
    prefix = 'emr restore-from-hbase-backup'
    steps = [
        {
            'HadoopJarStep': {
                'Args': [
                    'emr.hbase.backup.Main',
                    '--restore',
                    '--backup-dir',
                    's3://abc/',
                ],
                'Jar': '/home/hadoop/lib/hbase.jar',
            },
            'Name': 'Restore HBase',
            'ActionOnFailure': 'CANCEL_AND_WAIT',
        }
    ]

    def test_restore_from_hbase_backup(self):
        args = ' --cluster-id j-ABCD --dir s3://abc/'
        cmdline = self.prefix + args
        result = {'JobFlowId': 'j-ABCD', 'Steps': self.steps}

        self.assert_params_for_cmd(cmdline, result)

    def test_restore_from_hbase_backup_version(self):
        args = ' --cluster-id j-ABCD --dir s3://abc/ --backup-version DEF'
        cmdline = self.prefix + args

        steps = deepcopy(self.steps)
        steps[0]['HadoopJarStep']['Args'].append('--backup-version')
        steps[0]['HadoopJarStep']['Args'].append('DEF')
        result = {'JobFlowId': 'j-ABCD', 'Steps': steps}

        self.assert_params_for_cmd(cmdline, result)

    @mock.patch('awscli.customizations.emr.' 'emrutils.get_release_label')
    def test_unsupported_command_on_release_based_cluster_error(
        self, grl_patch
    ):
        grl_patch.return_value = 'emr-4.0'
        args = ' --cluster-id j-ABCD --dir s3://abc/'
        cmdline = self.prefix + args
        expected_error_msg = (
            "\naws: [ERROR]: An error occurred (ParamValidation): restore-from-hbase-backup"
            " is not supported with 'emr-4.0' release.\n"
        )
        result = self.run_cmd(cmdline, 252)

        self.assertEqual(result[1], expected_error_msg)


if __name__ == "__main__":
    unittest.main()



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import copy
import os

from botocore.compat import OrderedDict, json

from awscli.testutils import mock, unittest
from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)
from tests.unit.customizations.emr import test_constants as CONSTANTS
from tests.unit.customizations.emr import (
    test_constants_instance_fleets as CONSTANTS_FLEET,
)

DEFAULT_CLUSTER_NAME = "Development Cluster"

DEFAULT_INSTANCE_GROUPS_ARG = (
    'InstanceGroupType=MASTER,Name=MASTER,'
    'InstanceCount=1,InstanceType=m1.large '
    'InstanceGroupType=CORE,Name=CORE,'
    'InstanceCount=1,InstanceType=m1.large '
    'InstanceGroupType=TASK,Name=TASK,'
    'InstanceCount=1,InstanceType=m1.large '
)

HA_INSTANCE_GROUPS_ARG = (
    'InstanceGroupType=MASTER,Name=MASTER,'
    'InstanceCount=3,InstanceType=m1.large '
    'InstanceGroupType=CORE,Name=CORE,'
    'InstanceCount=1,InstanceType=m1.large '
)

DEFAULT_INSTANCE_GROUPS = [
    {
        'InstanceRole': 'MASTER',
        'InstanceCount': 1,
        'Name': 'MASTER',
        'Market': 'ON_DEMAND',
        'InstanceType': 'm1.large',
    },
    {
        'InstanceRole': 'CORE',
        'InstanceCount': 1,
        'Name': 'CORE',
        'Market': 'ON_DEMAND',
        'InstanceType': 'm1.large',
    },
    {
        'InstanceRole': 'TASK',
        'InstanceCount': 1,
        'Name': 'TASK',
        'Market': 'ON_DEMAND',
        'InstanceType': 'm1.large',
    },
]

HA_INSTANCE_GROUPS = [
    {
        'InstanceRole': 'MASTER',
        'InstanceCount': 3,
        'Name': 'MASTER',
        'Market': 'ON_DEMAND',
        'InstanceType': 'm1.large',
    },
    {
        'InstanceRole': 'CORE',
        'InstanceCount': 1,
        'Name': 'CORE',
        'Market': 'ON_DEMAND',
        'InstanceType': 'm1.large',
    },
]


DEFAULT_CMD = (
    'emr create-cluster --release-label emr-4.0.0'
    ' --use-default-roles'
    ' --instance-groups ' + DEFAULT_INSTANCE_GROUPS_ARG + ' '
)

DEFAULT_INSTANCES = {
    'KeepJobFlowAliveWhenNoSteps': True,
    'TerminationProtected': False,
    'InstanceGroups': DEFAULT_INSTANCE_GROUPS,
}

HA_INSTANCES = {
    'KeepJobFlowAliveWhenNoSteps': True,
    'TerminationProtected': False,
    'InstanceGroups': HA_INSTANCE_GROUPS,
}

EC2_ROLE_NAME = "EMR_EC2_DefaultRole"
EMR_ROLE_NAME = "EMR_DefaultRole"

DEFAULT_KERBEROS_ATTRIBUTES_ARGS = (
    'Realm=EC2.INTERNAL,'
    'KdcAdminPassword=123,'
    'CrossRealmTrustPrincipalPassword=123,'
    'ADDomainJoinUser=aws,'
    'ADDomainJoinPassword=123'
)

KERBEROS_ATTRIBUTES = {
    'Realm': 'EC2.INTERNAL',
    'KdcAdminPassword': '123',
    'CrossRealmTrustPrincipalPassword': '123',
    'ADDomainJoinUser': 'aws',
    'ADDomainJoinPassword': '123',
}

TEST_BA = [
    {
        'ScriptBootstrapAction': {
            'Path': 's3://test/ba1',
            'Args': ['arg1', 'arg2', 'arg3'],
        },
        'Name': 'ba1',
    },
    {
        'ScriptBootstrapAction': {
            'Path': 's3://test/ba2',
            'Args': ['arg1', 'arg2', 'arg3'],
        },
        'Name': 'ba2',
    },
]

CUSTOM_JAR_STEP = {
    'Name': 'Custom JAR',
    'ActionOnFailure': 'CONTINUE',
    'HadoopJarStep': {'Jar': 's3://mybucket/mytest.jar'},
}

STREAMING_ARGS = (
    'Args=-files,'
    's3://elasticmapreduce/samples/wordcount/wordSplitter.py,'
    '-mapper,wordSplitter.py,'
    '-reducer,aggregate,'
    '-input,s3://elasticmapreduce/samples/wordcount/input,'
    '-output,s3://mybucket/wordcount/output/2014-04-18/12-15-24'
)

STREAMING_HADOOP_JAR_STEP = {
    'Jar': 'command-runner.jar',
    'Args': [
        'hadoop-streaming',
        '-files',
        's3://elasticmapreduce/samples/wordcount/wordSplitter.py',
        '-mapper',
        'wordSplitter.py',
        '-reducer',
        'aggregate',
        '-input',
        's3://elasticmapreduce/samples/wordcount/input',
        '-output',
        's3://mybucket/wordcount/output/2014-04-18/12-15-24',
    ],
}

CREATE_CLUSTER_RESULT = {
    "JobFlowId": "j-XXXX",
    "ClusterArn": "arn:aws:elasticmapreduce:region:012345678910:cluster/j-XXXX",
}

CONSTRUCTED_RESULT = {
    "ClusterId": "j-XXXX",
    "ClusterArn": "arn:aws:elasticmapreduce:region:012345678910:cluster/j-XXXX",
}

DEFAULT_RESULT = {
    'Name': DEFAULT_CLUSTER_NAME,
    'Instances': DEFAULT_INSTANCES,
    'ReleaseLabel': 'emr-4.0.0',
    'VisibleToAllUsers': True,
    'JobFlowRole': EC2_ROLE_NAME,
    'ServiceRole': EMR_ROLE_NAME,
    'Tags': [],
}

EMR_MANAGED_MASTER_SECURITY_GROUP = 'sg-master1'

EMR_MANAGED_SLAVE_SECURITY_GROUP = 'sg-slave1'

SERVICE_ACCESS_SECURITY_GROUP = "sg-service-access"

ADDITIONAL_MASTER_SECURITY_GROUPS = [
    'sg-addMaster1',
    'sg-addMaster2',
    'sg-addMaster3',
    'sg-addMaster4',
]

ADDITIONAL_SLAVE_SECURITY_GROUPS = [
    'sg-addSlave1',
    'sg-addSlave2',
    'sg-addSlave3',
    'sg-addSlave4',
]

HIVE_BASIC_ARGS = (
    'Args=-f,s3://elasticmapreduce/samples/hive-ads/libs/model-build.q'
)

HIVE_DEFAULT_STEP = {
    'Name': 'Hive program',
    'ActionOnFailure': 'CONTINUE',
    'HadoopJarStep': {
        'Jar': ('command-runner.jar'),
        'Args': [
            'hive-script',
            '--run-hive-script',
            '--args',
            '-f',
            's3://elasticmapreduce/samples/hive-ads/libs/model-build.q',
        ],
    },
}

HIVE_STEP = {
    'Name': 'HiveBasicStep',
    'ActionOnFailure': 'CANCEL_AND_WAIT',
    'HadoopJarStep': {
        'Jar': ('command-runner.jar'),
        'Args': [
            'hive-script',
            '--run-hive-script',
            '--args',
            '-f',
            's3://elasticmapreduce/samples/hive-ads/libs/model-build.q',
        ],
    },
}

PIG_BASIC_ARGS = (
    'Args=-f,' + 's3://elasticmapreduce/samples/pig-apache/do-reports2.pig'
)

PIG_DEFAULT_STEP = {
    'Name': 'Pig program',
    'ActionOnFailure': 'CONTINUE',
    'HadoopJarStep': {
        'Jar': ('command-runner.jar'),
        'Args': [
            'pig-script',
            '--run-pig-script',
            '--args',
            '-f',
            's3://elasticmapreduce/samples/' 'pig-apache/do-reports2.pig',
        ],
    },
}

PIG_STEP = {
    'Name': 'PigBasicStep',
    'ActionOnFailure': 'CANCEL_AND_WAIT',
    'HadoopJarStep': {
        'Jar': ('command-runner.jar'),
        'Args': [
            'pig-script',
            '--run-pig-script',
            '--args',
            '-f',
            's3://elasticmapreduce/samples/' 'pig-apache/do-reports2.pig',
        ],
    },
}


class TestCreateCluster(BaseAWSCommandParamsTest):
    prefix = 'emr create-cluster '

    def test_ami_version_release_label_exclusive_validation(self):
        cmd = (
            self.prefix + '--ami-version 3.8.0 --release-label '
            'emr-4.0.0 --instance-groups ' + DEFAULT_INSTANCE_GROUPS_ARG
        )
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): You cannot specify both --ami-version'
            ' and --release-label options together.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_if_ami_version_or_release_label_is_provided(self):
        cmd = self.prefix + ' --instance-groups ' + DEFAULT_INSTANCE_GROUPS_ARG
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): Either --ami-version or'
            ' --release-label is required.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_no_special_steps_added_for_applications(self):
        cmd = DEFAULT_CMD + '--applications Name=MapR'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Applications'] = [{'Name': 'MapR'}]
        self.assert_params_for_cmd(cmd, result)

        cmd = DEFAULT_CMD + '--applications Name=HUE'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Applications'] = [{'Name': 'HUE'}]
        self.assert_params_for_cmd(cmd, result)

        cmd = DEFAULT_CMD + '--applications Name=HIVE'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Applications'] = [{'Name': 'HIVE'}]
        self.assert_params_for_cmd(cmd, result)

        cmd = DEFAULT_CMD + '--applications Name=PIG'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Applications'] = [{'Name': 'PIG'}]
        self.assert_params_for_cmd(cmd, result)

        cmd = DEFAULT_CMD + '--applications Name=HBASE'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Applications'] = [{'Name': 'HBASE'}]
        self.assert_params_for_cmd(cmd, result)

        cmd = DEFAULT_CMD + '--applications Name=IMPALA'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Applications'] = [{'Name': 'IMPALA'}]
        self.assert_params_for_cmd(cmd, result)

        cmd = DEFAULT_CMD + '--applications Name=GANGLIA'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Applications'] = [{'Name': 'GANGLIA'}]
        self.assert_params_for_cmd(cmd, result)

    def test_configurations(self):
        cmd = (
            DEFAULT_CMD + ' --configurations '
            '[{"Classification":"hadoop-env","Properties":{'
            '"someProperty":"someValue"},"Configurations":[]}]'
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Configurations'] = [
            {
                'Classification': 'hadoop-env',
                'Configurations': [],
                'Properties': {'someProperty': 'someValue'},
            }
        ]
        self.assert_params_for_cmd(cmd, result)

    def test_quick_start(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.0.0 --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-4.0.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def assert_error_message_has_field_name(self, error_msg, field_name):
        self.assertIn('Missing required parameter', error_msg)
        self.assertIn(field_name, error_msg)

    def test_default_cmd(self):
        self.assert_params_for_cmd(DEFAULT_CMD, DEFAULT_RESULT)

    def test_cluster_without_service_role_and_instance_profile(self):
        cmd = (
            'emr create-cluster --release-label emr-4.0.0 '
            '--instance-groups ' + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        del result['JobFlowRole']
        del result['ServiceRole']
        self.assert_params_for_cmd(cmd, result)

    def test_cluster_with_service_role_and_instance_profile(self):
        cmd = (
            'emr create-cluster --release-label emr-4.0.0'
            ' --service-role ServiceRole --ec2-attributes '
            'InstanceProfile=Ec2_InstanceProfile '
            '--instance-groups ' + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['JobFlowRole'] = 'Ec2_InstanceProfile'
        result['ServiceRole'] = 'ServiceRole'
        self.assert_params_for_cmd(cmd, result)

    def test_mutual_exclusive_use_default_roles_and_service_role(self):
        cmd = (
            DEFAULT_CMD
            + '--ec2-attributes InstanceProfile=Ec2_InstanceProfile'
        )
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): You cannot specify both --use-default-roles '
            'and --ec2-attributes InstanceProfile options together. Either '
            'choose --use-default-roles or use both --service-role <roleName>'
            ' and --ec2-attributes InstanceProfile=<profileName>.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_mutual_exclusive_use_default_roles_and_instance_profile(self):
        cmd = (
            DEFAULT_CMD + '--service-role ServiceRole '
            '--ec2-attributes InstanceProfile=Ec2_InstanceProfile'
        )
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): You cannot specify both --use-default-roles '
            'and --service-role options together. Either choose '
            '--use-default-roles or use both --service-role <roleName> '
            'and --ec2-attributes InstanceProfile=<profileName>.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_cluster_name_no_space(self):
        cmd = DEFAULT_CMD + '--name MyCluster'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Name'] = 'MyCluster'
        self.assert_params_for_cmd(cmd, result)

    def test_cluster_name_with_space(self):
        cmd = DEFAULT_CMD.split() + ['--name', 'My Cluster']
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Name'] = 'My Cluster'
        self.assert_params_for_cmd(cmd, result)

    def test_ami_version(self):
        cmd = DEFAULT_CMD + '--release-label emr-4.0.0'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['ReleaseLabel'] = 'emr-4.0.0'
        self.assert_params_for_cmd(cmd, result)

    def test_log_uri(self):
        test_log_uri = 's3://test/logs'
        cmd = DEFAULT_CMD + '--log-uri ' + test_log_uri
        result = copy.deepcopy(DEFAULT_RESULT)
        result['LogUri'] = test_log_uri
        self.assert_params_for_cmd(cmd, result)

    def test_additional_info(self):
        test_info = '{ami32: "ami-82e305f5"}'
        cmd = DEFAULT_CMD.split() + ['--additional-info', test_info]
        result = copy.deepcopy(DEFAULT_RESULT)
        result['AdditionalInfo'] = test_info
        self.assert_params_for_cmd(cmd, result)

    def test_auto_terminte(self):
        cmd = (
            'emr create-cluster --use-default-roles'
            ' --release-label emr-4.0.0 '
            '--auto-terminate --instance-groups ' + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        instances = copy.deepcopy(DEFAULT_INSTANCES)
        instances['KeepJobFlowAliveWhenNoSteps'] = False
        result['Instances'] = instances
        self.assert_params_for_cmd(cmd, result)

    def test_auto_terminate_and_no_auto_terminate(self):
        cmd = (
            DEFAULT_CMD
            + '--release-label emr-4.0.0 '
            + '--auto-terminate --no-auto-terminate'
        )
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): cannot use both --no-auto-terminate and'
            ' --auto-terminate options together.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_termination_protected(self):
        cmd = DEFAULT_CMD + '--termination-protected'
        result = copy.deepcopy(DEFAULT_RESULT)
        instances = copy.deepcopy(DEFAULT_INSTANCES)
        instances['TerminationProtected'] = True
        result['Instances'] = instances
        self.assert_params_for_cmd(cmd, result)

    def test_no_termination_protected(self):
        cmd = DEFAULT_CMD + '--no-termination-protected'
        self.assert_params_for_cmd(cmd, DEFAULT_RESULT)

    def test_termination_protected_and_no_termination_protected(self):
        cmd = (
            DEFAULT_CMD + '--termination-protected --no-termination-protected'
        )
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): cannot use both --termination-protected'
            ' and --no-termination-protected options together.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_unhealthy_node_replacement(self):
        cmd = DEFAULT_CMD + '--unhealthy-node-replacement'
        result = copy.deepcopy(DEFAULT_RESULT)
        instances = copy.deepcopy(DEFAULT_INSTANCES)
        instances['UnhealthyNodeReplacement'] = True
        result['Instances'] = instances
        self.assert_params_for_cmd(cmd, result)

    def test_no_unhealthy_node_replacement(self):
        cmd = DEFAULT_CMD + '--no-unhealthy-node-replacement'
        result = copy.deepcopy(DEFAULT_RESULT)
        instances = copy.deepcopy(DEFAULT_INSTANCES)
        instances['UnhealthyNodeReplacement'] = False
        result['Instances'] = instances
        self.assert_params_for_cmd(cmd, result)

    def test_unhealthy_node_replacement_and_no_unhealthy_node_replacement(
        self,
    ):
        cmd = (
            DEFAULT_CMD
            + '--unhealthy-node-replacement --no-unhealthy-node-replacement'
        )
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): cannot use both --unhealthy-node-replacement'
            ' and --no-unhealthy-node-replacement options together.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_visible_to_all_users(self):
        cmd = DEFAULT_CMD + '--visible-to-all-users'
        self.assert_params_for_cmd(cmd, DEFAULT_RESULT)

    def test_no_visible_to_all_users(self):
        cmd = DEFAULT_CMD + '--no-visible-to-all-users'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['VisibleToAllUsers'] = False
        self.assert_params_for_cmd(cmd, result)

    def test_visible_to_all_users_and_no_visible_to_all_users(self):
        cmd = DEFAULT_CMD + '--visible-to-all-users --no-visible-to-all-users'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): cannot use both --visible-to-all-users and '
            '--no-visible-to-all-users options together.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_extended_support(self):
        cmd = DEFAULT_CMD + '--extended-support'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['ExtendedSupport'] = True
        self.assert_params_for_cmd(cmd, result)

    def test_no_extended_support(self):
        cmd = DEFAULT_CMD + '--no-extended-support'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['ExtendedSupport'] = False
        self.assert_params_for_cmd(cmd, result)

    def test_extended_support_and_no_extended_support(self):
        cmd = DEFAULT_CMD + '--extended-support --no-extended-support'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): cannot use both --extended-support'
            ' and --no-extended-support options together.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_tags(self):
        cmd = DEFAULT_CMD.split() + ['--tags', 'k1=v1', 'k2', 'k3=spaces  v3']
        result = copy.deepcopy(DEFAULT_RESULT)
        tags = [
            {'Key': 'k1', 'Value': 'v1'},
            {'Key': 'k2', 'Value': ''},
            {'Key': 'k3', 'Value': 'spaces  v3'},
        ]
        result['Tags'] = tags
        self.assert_params_for_cmd(cmd, result)

    def test_enable_debugging(self):
        cmd = DEFAULT_CMD + '--log-uri s3://test/logs --enable-debugging'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['LogUri'] = 's3://test/logs'
        debugging_config = [
            {
                'Name': 'Setup Hadoop Debugging',
                'ActionOnFailure': 'TERMINATE_CLUSTER',
                'HadoopJarStep': {
                    'Args': ['state-pusher-script'],
                    'Jar': 'command-runner.jar',
                },
            }
        ]
        result['Steps'] = debugging_config
        self.assert_params_for_cmd(cmd, result)

        cmd = DEFAULT_CMD + (
            '--log-uri s3://test/logs --enable-debugging ' '--region us-west-2'
        )
        debugging_config = [
            {
                'Name': 'Setup Hadoop Debugging',
                'ActionOnFailure': 'TERMINATE_CLUSTER',
                'HadoopJarStep': {
                    'Args': ['state-pusher-script'],
                    'Jar': 'command-runner.jar',
                },
            }
        ]
        result['Steps'] = debugging_config
        self.assert_params_for_cmd(cmd, result)

    def test_enable_debugging_no_log_uri(self):
        cmd = DEFAULT_CMD + '--enable-debugging'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): LogUri not specified. You must specify a logUri'
            ' if you enable debugging when creating a cluster.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_enable_debugging_and_no_enable_debugging(self):
        cmd = (
            DEFAULT_CMD
            + '--enable-debugging --no-enable-debugging'
            + ' --log-uri s3://test/logs'
        )
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): cannot use both --enable-debugging and '
            '--no-enable-debugging options together.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_instance_groups_default_name_market(self):
        cmd = (
            'emr create-cluster --use-default-roles --release-label emr-4.0.0 '
            '--instance-groups '
            'InstanceGroupType=MASTER,InstanceCount=1,InstanceType=m1.large '
            'InstanceGroupType=CORE,InstanceCount=1,InstanceType=m1.large '
            'InstanceGroupType=TASK,InstanceCount=1,InstanceType=m1.large '
        )
        self.assert_params_for_cmd(cmd, DEFAULT_RESULT)

    def test_instance_groups_instance_group_type_mismatch_cases(self):
        cmd = (
            'emr create-cluster --use-default-roles --release-label emr-4.0.0 '
            '--instance-groups '
            'Name=MASTER,InstanceGroupType=MaSter,InstanceCount=1,'
            'InstanceType=m1.large Name=CORE,InstanceGroupType=cORE,'
            'InstanceCount=1,InstanceType=m1.large Name=TASK,'
            'InstanceGroupType=tAsK,InstanceCount=1,InstanceType=m1.large'
        )
        self.assert_params_for_cmd(cmd, DEFAULT_RESULT)

    def test_instance_groups_instance_type_and_count(self):
        cmd = (
            'emr create-cluster --use-default-roles --release-label emr-4.0.0 '
            '--instance-type m1.large'
        )
        expected_result = copy.deepcopy(DEFAULT_RESULT)
        expected_result['Instances'] = {
            'KeepJobFlowAliveWhenNoSteps': True,
            'TerminationProtected': False,
            'InstanceGroups': [
                {
                    'InstanceRole': 'MASTER',
                    'InstanceCount': 1,
                    'Name': 'MASTER',
                    'Market': 'ON_DEMAND',
                    'InstanceType': 'm1.large',
                }
            ],
        }
        self.assert_params_for_cmd(cmd, expected_result)
        cmd = (
            'emr create-cluster --use-default-roles --release-label emr-4.0.0 '
            '--instance-type m1.large --instance-count 3'
        )
        expected_result = copy.deepcopy(DEFAULT_RESULT)
        expected_result['Instances'] = {
            'KeepJobFlowAliveWhenNoSteps': True,
            'TerminationProtected': False,
            'InstanceGroups': [
                {
                    'InstanceRole': 'MASTER',
                    'InstanceCount': 1,
                    'Name': 'MASTER',
                    'Market': 'ON_DEMAND',
                    'InstanceType': 'm1.large',
                },
                {
                    'InstanceRole': 'CORE',
                    'InstanceCount': 2,
                    'Name': 'CORE',
                    'Market': 'ON_DEMAND',
                    'InstanceType': 'm1.large',
                },
            ],
        }
        self.assert_params_for_cmd(cmd, expected_result)

    def test_instance_groups_missing_required_parameter_error(self):
        cmd = (
            'emr create-cluster --use-default-roles --release-label'
            ' emr-4.0.0 '
        )
        expect_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): Must specify either --instance-groups or '
            '--instance-type with --instance-count(optional) to '
            'configure instance groups.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expect_error_msg, result[1])

        cmd = (
            'emr create-cluster --use-default-roles --release-label emr-4.0.0 '
            '--instance-count 2'
        )
        expect_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): Must specify either --instance-groups or '
            '--instance-type with --instance-count(optional) to '
            'configure instance groups.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expect_error_msg, result[1])

    def test_instance_groups_exclusive_parameter_validation_error(self):
        cmd = (
            'emr create-cluster --use-default-roles --release-label 4.0.0 '
            '--instance-type m1.large --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        expect_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): You may not specify --instance-type '
            'or --instance-count with --instance-groups, '
            'because --instance-type and --instance-count are '
            'shortcut options for --instance-groups.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expect_error_msg, result[1])

        cmd = (
            'emr create-cluster --use-default-roles --release-label emr-4.0.0 '
            '--instance-type m1.large --instance-count 2 '
            '--instance-groups ' + DEFAULT_INSTANCE_GROUPS_ARG
        )
        expect_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): You may not specify --instance-type '
            'or --instance-count with --instance-groups, '
            'because --instance-type and --instance-count are '
            'shortcut options for --instance-groups.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expect_error_msg, result[1])

    def test_instance_groups_missing_instance_group_type_error(self):
        cmd = (
            'emr create-cluster --use-default-roles --release-label emr-4.0.0 '
            '--auto-terminate '
            '--instance-groups '
            'Name=Master,InstanceCount=1,InstanceType=m1.small'
        )
        stderr = self.run_cmd(cmd, 252)[1]
        self.assert_error_message_has_field_name(stderr, 'InstanceGroupType')

    def test_instance_groups_missing_instance_type_error(self):
        cmd = (
            'emr create-cluster --use-default-roles --release-label emr-4.0.0 '
            '--auto-terminate '
            '--instance-groups '
            'Name=Master,InstanceGroupType=MASTER,InstanceCount=1'
        )
        stderr = self.run_cmd(cmd, 252)[1]
        self.assert_error_message_has_field_name(stderr, 'InstanceType')

    def test_instance_groups_missing_instance_count_error(self):
        cmd = (
            'emr create-cluster --use-default-roles --release-label emr-4.0.0 '
            '--auto-terminate '
            '--instance-groups '
            'Name=Master,InstanceGroupType=MASTER,InstanceType=m1.xlarge'
        )
        stderr = self.run_cmd(cmd, 252)[1]
        self.assert_error_message_has_field_name(stderr, 'InstanceCount')

    def test_instance_groups_from_json_file(self):
        data_path = os.path.join(
            os.path.dirname(__file__), 'input_instance_groups.json'
        )
        cmd = (
            'emr create-cluster --use-default-roles'
            ' --release-label emr-4.0.0  '
            '--instance-groups file://' + data_path
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Instances']['InstanceGroups'] = [
            {
                'InstanceRole': 'MASTER',
                'InstanceCount': 1,
                'Name': 'Master Instance Group',
                'Market': 'ON_DEMAND',
                'InstanceType': 'm1.large',
            },
            {
                'InstanceRole': 'CORE',
                'InstanceCount': 2,
                'Name': 'Core Instance Group',
                'Market': 'ON_DEMAND',
                'InstanceType': 'm1.xlarge',
            },
            {
                'InstanceRole': 'TASK',
                'InstanceCount': 3,
                'Name': 'Task Instance Group',
                'Market': 'SPOT',
                'BidPrice': '3.45',
                'InstanceType': 'm1.xlarge',
            },
        ]
        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_from_json_file_spot_bidprice_equals_ondemandprice(
        self,
    ):
        data_path = os.path.join(
            os.path.dirname(__file__),
            'input_instance_groups_spot_bidprice_equals_ondemandprice.json',
        )
        cmd = (
            'emr create-cluster --use-default-roles'
            ' --release-label emr-4.0.0  '
            '--instance-groups file://' + data_path
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Instances']['InstanceGroups'] = [
            {
                'InstanceRole': 'MASTER',
                'InstanceCount': 1,
                'Name': 'Master Instance Group',
                'Market': 'SPOT',
                'InstanceType': 'm1.large',
            },
            {
                'InstanceRole': 'CORE',
                'InstanceCount': 2,
                'Name': 'Core Instance Group',
                'Market': 'SPOT',
                'InstanceType': 'm1.xlarge',
            },
            {
                'InstanceRole': 'TASK',
                'InstanceCount': 3,
                'Name': 'Task Instance Group',
                'Market': 'SPOT',
                'InstanceType': 'm1.xlarge',
            },
        ]
        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_adds_configurations(self):
        data_path = os.path.join(
            os.path.dirname(__file__),
            'input_instance_groups_with_configurations.json',
        )
        cmd = (
            'emr create-cluster --use-default-roles'
            ' --release-label emr-4.0.0 '
            '--instance-groups file://' + data_path
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Instances']['InstanceGroups'][1]['Configurations'] = [
            OrderedDict(
                [
                    ("Classification", "hdfs-site"),
                    (
                        "Properties",
                        OrderedDict(
                            [
                                ("test-key1", "test-value1"),
                                ("test-key2", "test-value2"),
                            ]
                        ),
                    ),
                ]
            )
        ]
        self.assert_params_for_cmd(cmd, result)

    def test_ec2_attributes_no_az(self):
        cmd = (
            'emr create-cluster --release-label emr-4.0.0 '
            '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
            + ' --ec2-attributes KeyName=testkey,SubnetId=subnet-123456,'
            'InstanceProfile=EMR_EC2_DefaultRole '
            '--service-role EMR_DefaultRole'
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Instances']['Ec2KeyName'] = 'testkey'
        result['Instances']['Ec2SubnetId'] = 'subnet-123456'
        result['JobFlowRole'] = 'EMR_EC2_DefaultRole'
        self.assert_params_for_cmd(cmd, result)

    def test_ec2_attributes_az(self):
        cmd = DEFAULT_CMD + '--ec2-attributes AvailabilityZone=us-east-1a'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Instances']['Placement'] = {'AvailabilityZone': 'us-east-1a'}
        self.assert_params_for_cmd(cmd, result)

    def test_ec2_attributes_subnet_az_error(self):
        cmd = (
            DEFAULT_CMD
            + '--ec2-attributes '
            + 'SubnetId=subnet-123456,AvailabilityZone=us-east-1a'
        )
        expect_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): You may not specify both a SubnetId and an Availab'
            'ilityZone (placement) because ec2SubnetId implies a placement.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expect_error_msg, result[1])

    def test_ec2_attributes_with_subnet_from_json_file(self):
        data_path = os.path.join(
            os.path.dirname(__file__), 'input_ec2_attributes_with_subnet.json'
        )
        cmd = (
            'emr create-cluster --release-label emr-4.0.0 '
            '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
            + ' --ec2-attributes file://'
            + data_path
            + ' --service-role EMR_DefaultRole'
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Instances']['Ec2KeyName'] = 'testkey'
        result['Instances']['Ec2SubnetId'] = 'subnet-123456'
        result['JobFlowRole'] = 'EMR_EC2_DefaultRole'
        self.assert_params_for_cmd(cmd, result)

    def test_ec2_attributes_with_az_from_json_file(self):
        data_path = os.path.join(
            os.path.dirname(__file__), 'input_ec2_attributes_with_az.json'
        )
        cmd = (
            'emr create-cluster --release-label emr-4.0.0 '
            '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
            + ' --ec2-attributes file://'
            + data_path
            + ' --service-role EMR_DefaultRole'
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Instances']['Ec2KeyName'] = 'testkey'
        result['Instances']['Placement'] = {'AvailabilityZone': 'us-east-1a'}
        result['JobFlowRole'] = 'EMR_EC2_DefaultRole'
        self.assert_params_for_cmd(cmd, result)

    # Bootstrap Actions test cases
    def test_bootstrap_actions_missing_path_error(self):
        cmd = DEFAULT_CMD + '--bootstrap-actions Name=ba1,Args=arg1,arg2'
        stderr = self.run_cmd(cmd, 252)[1]
        self.assert_error_message_has_field_name(stderr, 'Path')

    def test_bootstrap_actions_with_all_fields(self):
        cmd = DEFAULT_CMD + (
            ' --bootstrap-actions '
            'Path=s3://test/ba1,Name=ba1,Args=arg1,arg2,arg3 '
            'Path=s3://test/ba2,Name=ba2,Args=arg1,arg2,arg3'
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['BootstrapActions'] = TEST_BA

        self.assert_params_for_cmd(cmd, result)

    def test_bootstrap_actions_exceed_maximum_error(self):
        cmd = DEFAULT_CMD + ' --bootstrap-actions'
        ba_cmd = ' Path=s3://test/ba1,Name=ba1,Args=arg1,arg2,arg3'
        for i in range(1, 18):
            cmd += ba_cmd

        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): maximum number of '
            + 'bootstrap actions for a cluster exceeded.\n'
        )
        result = self.run_cmd(cmd, 252)

        self.assertEqual(expected_error_msg, result[1])

    def test_bootstrap_actions_exceed_maximum_with_applications_error(self):
        cmd = DEFAULT_CMD + ' --bootstrap-actions'
        ba_cmd = ' Path=s3://test/ba1,Name=ba1,Args=arg1,arg2,arg3'
        for i in range(1, 20):
            cmd += ba_cmd
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): maximum number of '
            + 'bootstrap actions for a cluster exceeded.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_boostrap_actions_with_default_fields(self):
        cmd = DEFAULT_CMD + (
            ' --bootstrap-actions Path=s3://test/ba1 Path=s3://test/ba2'
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['BootstrapActions'] = [
            {
                'Name': 'Bootstrap action',
                'ScriptBootstrapAction': {'Path': 's3://test/ba1'},
            },
            {
                'Name': 'Bootstrap action',
                'ScriptBootstrapAction': {'Path': 's3://test/ba2'},
            },
        ]
        self.assert_params_for_cmd(cmd, result)

    def test_bootstrap_actions_from_json_file(self):
        data_path = os.path.join(
            os.path.dirname(__file__), 'input_bootstrap_actions.json'
        )
        cmd = DEFAULT_CMD + ' --bootstrap-actions file://' + data_path
        result = copy.deepcopy(DEFAULT_RESULT)
        result['BootstrapActions'] = [
            {
                "Name": "Bootstrap Action 1",
                "ScriptBootstrapAction": {
                    "Path": "s3://mybucket/test1",
                    "Args": ["arg1", "arg2"],
                },
            },
            {
                "Name": "Bootstrap Action 2",
                "ScriptBootstrapAction": {
                    "Path": "s3://mybucket/test2",
                    "Args": ["arg1", "arg2"],
                },
            },
        ]
        self.assert_params_for_cmd(cmd, result)

    # Steps test cases
    def test_wrong_step_type_error(self):
        cmd = DEFAULT_CMD + '--steps Type=unknown'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The step type unknown is not supported.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_default_step_type_name_action_on_failure(self):
        cmd = DEFAULT_CMD + '--steps Jar=s3://mybucket/mytest.jar'
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Steps'] = [CUSTOM_JAR_STEP]
        self.assert_params_for_cmd(cmd, result)

    def test_custom_jar_step_missing_jar(self):
        cmd = DEFAULT_CMD + '--steps Name=CustomJarMissingJar'
        expect_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The following '
            + 'required parameters are missing for CustomJARStepConfig: Jar.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expect_error_msg, result[1])

    def test_custom_jar_step_with_all_fields(self):
        cmd = (
            DEFAULT_CMD
            + '--steps '
            + (
                'Name=Custom,Type=Custom_JAR,'
                'Jar=s3://mybucket/mytest.jar,'
                'Args=arg1,arg2,MainClass=mymainclass,'
                'ActionOnFailure=TERMINATE_CLUSTER'
            )
        )
        expected_steps = [
            {
                'Name': 'Custom',
                'ActionOnFailure': 'TERMINATE_CLUSTER',
                'HadoopJarStep': {
                    'Jar': 's3://mybucket/mytest.jar',
                    'Args': ['arg1', 'arg2'],
                    'MainClass': 'mymainclass',
                },
            }
        ]
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Steps'] = expected_steps
        self.assert_params_for_cmd(cmd, result)

    def test_streaming_step_with_default_fields(self):
        cmd = DEFAULT_CMD + '--steps Type=Streaming,' + STREAMING_ARGS
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Steps'] = [
            {
                'Name': 'Streaming program',
                'ActionOnFailure': 'CONTINUE',
                'HadoopJarStep': STREAMING_HADOOP_JAR_STEP,
            }
        ]
        self.assert_params_for_cmd(cmd, result)

    def test_streaming_step_missing_args(self):
        cmd = DEFAULT_CMD + '--steps Type=Streaming'
        expect_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The following '
            + 'required parameters are missing for StreamingStepConfig: Args.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expect_error_msg, result[1])

    def test_streaming_jar_with_all_fields(self):
        test_step_config = (
            '--steps Type=Streaming,Name=StreamingStepAllFields,'
            'ActionOnFailure=CANCEL_AND_WAIT,' + STREAMING_ARGS
        )
        cmd = DEFAULT_CMD + test_step_config
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Steps'] = [
            {
                'Name': 'StreamingStepAllFields',
                'ActionOnFailure': 'CANCEL_AND_WAIT',
                'HadoopJarStep': STREAMING_HADOOP_JAR_STEP,
            }
        ]
        self.assert_params_for_cmd(cmd, result)

    def test_hive_step_with_default_fields(self):
        cmd = DEFAULT_CMD + (
            '--applications Name=Hive --steps Type=Hive,' + HIVE_BASIC_ARGS
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Applications'] = [{'Name': 'Hive'}]
        result['Steps'] = [HIVE_DEFAULT_STEP]
        self.assert_params_for_cmd(cmd, result)

    def test_hive_step_missing_args(self):
        cmd = DEFAULT_CMD + '--applications Name=Hive --steps Type=Hive'
        expect_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The following '
            + 'required parameters are missing for HiveStepConfig: Args.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expect_error_msg, result[1])

    def test_hive_step_with_all_fields(self):
        test_step_config = (
            'Type=Hive,ActionOnFailure=CANCEL_AND_WAIT,'
            'Name=HiveBasicStep,' + HIVE_BASIC_ARGS
        )
        cmd = DEFAULT_CMD + (
            '--applications Name=Hive --steps ' + test_step_config
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Steps'] = [HIVE_STEP]
        result['Applications'] = [{'Name': 'Hive'}]
        self.assert_params_for_cmd(cmd, result)

    def test_pig_step_with_default_fields(self):
        cmd = DEFAULT_CMD + (
            '--applications Name=Pig --steps Type=Pig,' + PIG_BASIC_ARGS
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Steps'] = [PIG_DEFAULT_STEP]
        result['Applications'] = [{'Name': 'Pig'}]
        self.assert_params_for_cmd(cmd, result)

    def test_pig_missing_args(self):
        cmd = DEFAULT_CMD + '--applications Name=Pig --steps Type=Pig'
        expect_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The following '
            + 'required parameters are missing for PigStepConfig: Args.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expect_error_msg, result[1])

    def test_pig_step_with_all_fields(self):
        test_step_config = (
            'Name=PigBasicStep,Type=Pig,'
            + PIG_BASIC_ARGS
            + ',ActionOnFailure=CANCEL_AND_WAIT'
        )
        cmd = DEFAULT_CMD + (
            '--applications Name=Pig --steps ' + test_step_config
        )
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Applications'] = [{'Name': 'Pig'}]
        result['Steps'] = [PIG_STEP]
        self.assert_params_for_cmd(cmd, result)

    @mock.patch('awscli.customizations.emr.emrutils.call')
    def test_constructed_result(self, call_patch):
        call_patch.return_value = CREATE_CLUSTER_RESULT
        cmd = DEFAULT_CMD
        result = self.run_cmd(cmd, expected_rc=0)
        result_json = json.loads(result[0])
        self.assertEqual(result_json, CONSTRUCTED_RESULT)

    def test_all_security_groups(self):
        cmd = DEFAULT_CMD + (
            '--ec2-attributes EmrManagedMasterSecurityGroup=sg-master1,'
            'EmrManagedSlaveSecurityGroup=sg-slave1,'
            'ServiceAccessSecurityGroup=sg-service-access,'
            'AdditionalMasterSecurityGroups='
            '[sg-addMaster1,sg-addMaster2,sg-addMaster3,'
            'sg-addMaster4],AdditionalSlaveSecurityGroups=[sg-addSlave1,'
            'sg-addSlave2,sg-addSlave3,sg-addSlave4]'
        )

        result = copy.deepcopy(DEFAULT_RESULT)
        instances = result['Instances']
        instances['EmrManagedMasterSecurityGroup'] = (
            EMR_MANAGED_MASTER_SECURITY_GROUP
        )
        instances['EmrManagedSlaveSecurityGroup'] = (
            EMR_MANAGED_SLAVE_SECURITY_GROUP
        )
        instances['ServiceAccessSecurityGroup'] = SERVICE_ACCESS_SECURITY_GROUP
        instances['AdditionalMasterSecurityGroups'] = (
            ADDITIONAL_MASTER_SECURITY_GROUPS
        )
        instances['AdditionalSlaveSecurityGroups'] = (
            ADDITIONAL_SLAVE_SECURITY_GROUPS
        )

        self.assert_params_for_cmd(cmd, result)

    def test_emr_managed_security_groups(self):
        cmd = DEFAULT_CMD + (
            '--ec2-attributes EmrManagedMasterSecurityGroup=sg-master1,'
            'EmrManagedSlaveSecurityGroup=sg-slave1,'
            'ServiceAccessSecurityGroup=sg-service-access'
        )

        result = copy.deepcopy(DEFAULT_RESULT)
        instances = result['Instances']
        instances['EmrManagedMasterSecurityGroup'] = (
            EMR_MANAGED_MASTER_SECURITY_GROUP
        )
        instances['EmrManagedSlaveSecurityGroup'] = (
            EMR_MANAGED_SLAVE_SECURITY_GROUP
        )
        instances['ServiceAccessSecurityGroup'] = SERVICE_ACCESS_SECURITY_GROUP

        self.assert_params_for_cmd(cmd, result)

    def test_additional_security_groups(self):
        cmd = DEFAULT_CMD + (
            '--ec2-attributes AdditionalMasterSecurityGroups=[sg-addMaster1'
            ',sg-addMaster2,sg-addMaster3,sg-addMaster4],AdditionalSlaveSecu'
            'rityGroups=[sg-addSlave1,sg-addSlave2,sg-addSlave3,sg-addSlave4]'
        )

        result = copy.deepcopy(DEFAULT_RESULT)
        instances = result['Instances']
        instances['AdditionalMasterSecurityGroups'] = (
            ADDITIONAL_MASTER_SECURITY_GROUPS
        )
        instances['AdditionalSlaveSecurityGroups'] = (
            ADDITIONAL_SLAVE_SECURITY_GROUPS
        )

        self.assert_params_for_cmd(cmd, result)

    def test_security_groups_from_json_file(self):
        data_path = os.path.join(
            os.path.dirname(__file__),
            'input_ec2_attributes_with_security_groups.json',
        )
        cmd = DEFAULT_CMD + '--ec2-attributes file://' + data_path

        result = copy.deepcopy(DEFAULT_RESULT)
        instances = result['Instances']
        instances['EmrManagedMasterSecurityGroup'] = (
            EMR_MANAGED_MASTER_SECURITY_GROUP
        )
        instances['EmrManagedSlaveSecurityGroup'] = (
            EMR_MANAGED_SLAVE_SECURITY_GROUP
        )
        instances['ServiceAccessSecurityGroup'] = SERVICE_ACCESS_SECURITY_GROUP
        instances['AdditionalMasterSecurityGroups'] = (
            ADDITIONAL_MASTER_SECURITY_GROUPS
        )
        instances['AdditionalSlaveSecurityGroups'] = (
            ADDITIONAL_SLAVE_SECURITY_GROUPS
        )

        self.assert_params_for_cmd(cmd, result)

    def test_instance_group_with_autoscaling_policy(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --auto-scaling-role EMR_AUTOSCALING_ROLE --instance-groups '
            + CONSTANTS.INSTANCE_GROUPS_WITH_AUTOSCALING_POLICY_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceGroups': CONSTANTS.INSTANCE_GROUPS_WITH_AUTOSCALING_POLICY,
            },
            'ReleaseLabel': 'emr-4.2.0',
            'AutoScalingRole': 'EMR_AUTOSCALING_ROLE',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_group_with_autoscaling_policy_missing_autoscaling_role(
        self,
    ):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-groups '
            + CONSTANTS.INSTANCE_GROUPS_WITH_AUTOSCALING_POLICY_ARG
        )
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): Must specify --auto-scaling-role when'
            ' configuring an AutoScaling policy for an instance group.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_scale_down_behavior(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.0.0 --scale-down-behavior TERMINATE_AT_INSTANCE_HOUR '
            '--instance-groups ' + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-4.0.0',
            'VisibleToAllUsers': True,
            'ScaleDownBehavior': 'TERMINATE_AT_INSTANCE_HOUR',
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_group_with_ebs_config(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-groups '
            + CONSTANTS.INSTANCE_GROUPS_WITH_EBS_VOLUME_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceGroups': CONSTANTS.INSTANCE_GROUPS_WITH_EBS,
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_with_ebs_config_missing_volume_type(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-groups '
            + CONSTANTS.INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_VOLTYPE_ARG
        )
        stderr = self.run_cmd(cmd, 252)[1]
        self.assert_error_message_has_field_name(stderr, 'VolumeType')

    def test_instance_groups_with_ebs_config_missing_size(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-groups '
            + CONSTANTS.INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_SIZE_ARG
        )
        stderr = self.run_cmd(cmd, 252)[1]
        self.assert_error_message_has_field_name(stderr, 'SizeInGB')

    def test_instance_groups_with_ebs_config_missing_volume_spec(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-groups '
            + CONSTANTS.INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_VOLSPEC_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceGroups': CONSTANTS.INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_VOLSPEC,
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_with_ebs_config_missing_iops(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-groups '
            + CONSTANTS.INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_IOPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceGroups': CONSTANTS.INSTANCE_GROUPS_WITH_EBS_VOLUME_MISSING_IOPS,
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_groups_with_ebs_config_multiple_instance_groups(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-groups '
            + CONSTANTS.MULTIPLE_INSTANCE_GROUPS_WITH_EBS_VOLUMES_VOLUME_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceGroups': CONSTANTS.MULTIPLE_INSTANCE_GROUPS_WITH_EBS_VOLUMES,
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_fleets_with_on_demand_master_only(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-fleets '
            + CONSTANTS_FLEET.INSTANCE_FLEETS_WITH_ON_DEMAND_MASTER_ONLY
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceFleets': CONSTANTS_FLEET.RES_INSTANCE_FLEETS_WITH_ON_DEMAND_MASTER_ONLY,
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_fleets_with_spot_master_only_with_ebs_conf(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-fleets '
            + CONSTANTS_FLEET.INSTANCE_FLEETS_WITH_SPOT_MASTER_ONLY_WITH_EBS_CONF
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceFleets': CONSTANTS_FLEET.RES_INSTANCE_FLEETS_WITH_SPOT_MASTER_ONLY_WITH_EBS_CONF,
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_fleets_with_spot_master_specific_azs(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-fleets '
            + CONSTANTS_FLEET.INSTANCE_FLEETS_WITH_SPOT_MASTER_ONLY
            + ' --ec2-attributes AvailabilityZones=[us-east-1a,us-east-1b]'
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceFleets': CONSTANTS_FLEET.RES_INSTANCE_FLEETS_WITH_SPOT_MASTER_ONLY,
                'Placement': {
                    'AvailabilityZones': ['us-east-1a', 'us-east-1b']
                },
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_fleets_with_spot_master_subnet_ids(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-fleets '
            + CONSTANTS_FLEET.INSTANCE_FLEETS_WITH_SPOT_MASTER_ONLY
            + ' --ec2-attributes SubnetIds=[subnetid-1,subnetid-2]'
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceFleets': CONSTANTS_FLEET.RES_INSTANCE_FLEETS_WITH_SPOT_MASTER_ONLY,
                'Ec2SubnetIds': ['subnetid-1', 'subnetid-2'],
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_fleets_with_spot_master_core_cluster_multiple_instance_types(
        self,
    ):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-fleets '
            + CONSTANTS_FLEET.INSTANCE_FLEETS_WITH_SPOT_MASTER_CORE_CLUSTER
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceFleets': CONSTANTS_FLEET.RES_INSTANCE_FLEETS_WITH_SPOT_MASTER_CORE_CLUSTER,
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_fleets_with_complex_config_from_json(self):
        data_path = os.path.join(
            os.path.dirname(__file__), 'input_instance_fleets.json'
        )
        cmd = (
            'emr create-cluster --use-default-roles --release-label emr-4.2.0  '
            '--instance-fleets file://' + data_path
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceFleets': CONSTANTS_FLEET.RES_INSTANCE_FLEETS_WITH_COMPLEX_CONFIG_FROM_JSON,
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'JobFlowRole': 'EMR_EC2_DefaultRole',
            'ServiceRole': 'EMR_DefaultRole',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_fleets_with_both_fleet_group_specified(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-fleets '
            + CONSTANTS_FLEET.INSTANCE_FLEETS_WITH_SPOT_MASTER_ONLY
            + ' --instance-groups '
            + CONSTANTS.INSTANCE_GROUPS_WITH_EBS_VOLUME_ARG
        )
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): You cannot specify both --instance-groups'
            ' and --instance-fleets options together.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_instance_fleets_with_both_subnetid_subnetids_specified(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-fleets '
            + CONSTANTS_FLEET.INSTANCE_FLEETS_WITH_SPOT_MASTER_ONLY
            + ' --ec2-attributes SubnetId=subnetid-1,SubnetIds=[subnetid-1,subnetid-2]'
        )
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): You cannot specify both SubnetId'
            ' and SubnetIds options together.\n'
        )
        result = self.run_cmd(cmd, 252)
        self.assertEqual(expected_error_msg, result[1])

    def test_create_cluster_with_security_config(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.7.2 --security-configuration MySecurityConfig '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-4.7.2',
            'VisibleToAllUsers': True,
            'Tags': [],
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_security_config_and_kerberos_attributes(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.7.2 --security-configuration MySecurityConfig'
            + ' --kerberos-attributes '
            + DEFAULT_KERBEROS_ATTRIBUTES_ARGS
            + ' --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'KerberosAttributes': KERBEROS_ATTRIBUTES,
            'ReleaseLabel': 'emr-4.7.2',
            'VisibleToAllUsers': True,
            'Tags': [],
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_custom_ami_id(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.7.2 --security-configuration MySecurityConfig '
            + ' --custom-ami-id ami-9be6f38c'
            + ' --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-4.7.2',
            'VisibleToAllUsers': True,
            'Tags': [],
            'CustomAmiId': 'ami-9be6f38c',
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_ebs_root_volume_size(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.7.2 --security-configuration MySecurityConfig '
            + ' --ebs-root-volume-size 50'
            + ' --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-4.7.2',
            'VisibleToAllUsers': True,
            'Tags': [],
            'EbsRootVolumeSize': 50,
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_ebs_root_volume_iops(self):
        cmd = (
            self.prefix
            + '--release-label emr-6.15.0 --security-configuration MySecurityConfig '
            + ' --ebs-root-volume-iops 3000'
            + ' --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-6.15.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'EbsRootVolumeIops': 3000,
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_ebs_root_volume_throughput(self):
        cmd = (
            self.prefix
            + '--release-label emr-6.15.0 --security-configuration MySecurityConfig '
            + ' --ebs-root-volume-throughput 125'
            + ' --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-6.15.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'EbsRootVolumeThroughput': 125,
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_repo_upgrade_on_boot(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.7.2 --security-configuration MySecurityConfig '
            + ' --repo-upgrade-on-boot NONE'
            + ' --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-4.7.2',
            'VisibleToAllUsers': True,
            'Tags': [],
            'RepoUpgradeOnBoot': 'NONE',
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_step_concurrency_level(self):
        cmd = (
            self.prefix
            + '--release-label emr-5.28.0 --security-configuration MySecurityConfig '
            + '--step-concurrency-level 30 '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-5.28.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'StepConcurrencyLevel': 30,
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_step_execution_role_arn(self):
        cmd = (
            self.prefix
            + '--release-label emr-6.7.0 '
            + '--step-execution-role-arn '
            + 'arn:aws:iam::123456789012:role/MyStepExecutionRole '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-6.7.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'StepExecutionRoleArn':
                'arn:aws:iam::123456789012:role/MyStepExecutionRole',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_managed_scaling_policy(self):
        cmd = (
            self.prefix
            + '--release-label emr-5.28.0 --security-configuration MySecurityConfig '
            + '--managed-scaling-policy ComputeLimits={MinimumCapacityUnits=2,MaximumCapacityUnits=4,UnitType=Instances,MaximumCoreCapacityUnits=1} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-5.28.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'ManagedScalingPolicy': {
                'ComputeLimits': {
                    'MinimumCapacityUnits': 2,
                    'MaximumCapacityUnits': 4,
                    'UnitType': 'Instances',
                    'MaximumCoreCapacityUnits': 1,
                }
            },
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_managed_scaling_policy_customer_knobs(self):
        cmd = (
            self.prefix
            + '--release-label emr-5.28.0 --security-configuration MySecurityConfig '
            + '--managed-scaling-policy ComputeLimits={MinimumCapacityUnits=2,MaximumCapacityUnits=4,'
            + 'UnitType=Instances,MaximumCoreCapacityUnits=1},ScalingStrategy=ADVANCED,'
            + 'UtilizationPerformanceIndex=1 --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-5.28.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'ManagedScalingPolicy': {
                'ComputeLimits': {
                    'MinimumCapacityUnits': 2,
                    'MaximumCapacityUnits': 4,
                    'UnitType': 'Instances',
                    'MaximumCoreCapacityUnits': 1,
                },
                'ScalingStrategy': 'ADVANCED',
                'UtilizationPerformanceIndex': 1,
            },
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_auto_termination_policy(self):
        cmd = (
            self.prefix
            + '--release-label emr-5.34.0 '
            + '--auto-termination-policy IdleTimeout=100 '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-5.34.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'AutoTerminationPolicy': {
                'IdleTimeout': 100,
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_monitoring_configuration(self):
        cmd = (
            self.prefix
            + '--release-label emr-5.34.0 '
            + '--monitoring-configuration '
            + 'CloudWatchLogConfiguration={Enabled=true,LogGroupName=MyLogGroup,'
            + 'LogStreamNamePrefix=MyPrefix,EncryptionKeyArn=arn:aws:kms:us-east-1:123456789012:key/12345678-1234-1234-1234-123456789012,'
            + 'LogTypes={STEP_LOGS=[STDOUT,STDERR],SPARK_DRIVER=[STDOUT],SPARK_EXECUTOR=[STDERR]}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-5.34.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'MonitoringConfiguration': {
                'CloudWatchLogConfiguration': {
                    'Enabled': True,
                    'LogGroupName': 'MyLogGroup',
                    'LogStreamNamePrefix': 'MyPrefix',
                    'EncryptionKeyArn': 'arn:aws:kms:us-east-1:123456789012:key/12345678-1234-1234-1234-123456789012',
                    'LogTypes': {
                        'STEP_LOGS': ['STDOUT', 'STDERR'],
                        'SPARK_DRIVER': ['STDOUT'],
                        'SPARK_EXECUTOR': ['STDERR'],
                    },
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_s3_logging_configuration(self):
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--log-uri s3://mybucket/logs '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={system-logs=emr-managed,application-logs=on-customer-s3only,persistent-ui-logs=disabled}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-7.13.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'LogUri': 's3://mybucket/logs',
            'MonitoringConfiguration': {
                'S3LoggingConfiguration': {
                    'LogTypeUploadPolicy': {
                        'system-logs': 'emr-managed',
                        'application-logs': 'on-customer-s3only',
                        'persistent-ui-logs': 'disabled',
                    },
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_cloudwatch_and_s3_logging_configuration(self):
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--log-uri s3://mybucket/logs '
            + '--monitoring-configuration '
            + 'CloudWatchLogConfiguration={Enabled=true,LogGroupName=MyLogGroup,'
            + 'LogStreamNamePrefix=MyPrefix,LogTypes={STEP_LOGS=[STDOUT,STDERR]}},'
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={system-logs=emr-managed,application-logs=on-customer-s3only}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-7.13.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'LogUri': 's3://mybucket/logs',
            'MonitoringConfiguration': {
                'CloudWatchLogConfiguration': {
                    'Enabled': True,
                    'LogGroupName': 'MyLogGroup',
                    'LogStreamNamePrefix': 'MyPrefix',
                    'LogTypes': {
                        'STEP_LOGS': ['STDOUT', 'STDERR'],
                    },
                },
                'S3LoggingConfiguration': {
                    'LogTypeUploadPolicy': {
                        'system-logs': 'emr-managed',
                        'application-logs': 'on-customer-s3only',
                    },
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_s3_logging_invalid_log_type(self):
        """Test that invalid log type raises error"""
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--log-uri s3://mybucket/logs '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={invalid-logs=emr-managed}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        self.assert_error_msg(cmd, 'InvalidS3LoggingLogTypeError', rc=252)

    def test_create_cluster_s3_logging_invalid_policy(self):
        """Test that invalid policy raises error"""
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--log-uri s3://mybucket/logs '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={system-logs=invalid-policy}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        self.assert_error_msg(cmd, 'InvalidS3LoggingPolicyError', rc=252)

    def test_create_cluster_s3_logging_on_customer_s3only_for_persistent_ui_logs(self):
        """Test that on-customer-s3only is not allowed for persistent-ui-logs"""
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--log-uri s3://mybucket/logs '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={persistent-ui-logs=on-customer-s3only}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        self.assert_error_msg(cmd, 'InvalidS3LoggingPersistentUiLogsPolicyError', rc=252)

    def test_create_cluster_s3_logging_requires_log_uri_for_system_logs(self):
        """Test that LogUri is required when system-logs uses emr-managed"""
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={system-logs=emr-managed,application-logs=disabled}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        self.assert_error_msg(cmd, 'S3LoggingConfigurationLogUriRequiredError', rc=252)

    def test_create_cluster_s3_logging_requires_log_uri_for_application_logs(self):
        """Test that LogUri is required when application-logs uses on-customer-s3only"""
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={system-logs=disabled,application-logs=on-customer-s3only}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        self.assert_error_msg(cmd, 'S3LoggingConfigurationLogUriRequiredError', rc=252)

    def test_create_cluster_s3_logging_requires_log_uri_for_both_logs(self):
        """Test that LogUri is required when both system-logs and application-logs use non-disabled policies"""
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={system-logs=emr-managed,application-logs=on-customer-s3only}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        self.assert_error_msg(cmd, 'S3LoggingConfigurationLogUriRequiredError', rc=252)

    def test_create_cluster_s3_logging_log_uri_not_allowed_when_both_disabled(self):
        """Test that LogUri is not allowed when both system-logs and application-logs are disabled"""
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--log-uri s3://mybucket/logs '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={system-logs=disabled,application-logs=disabled}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        self.assert_error_msg(cmd, 'S3LoggingConfigurationLogUriNotAllowedError', rc=252)

    def test_create_cluster_s3_logging_valid_with_log_uri(self):
        """Test valid S3LoggingConfiguration with LogUri"""
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--log-uri s3://mybucket/logs '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={system-logs=emr-managed,application-logs=on-customer-s3only,persistent-ui-logs=disabled}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-7.13.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'LogUri': 's3://mybucket/logs',
            'MonitoringConfiguration': {
                'S3LoggingConfiguration': {
                    'LogTypeUploadPolicy': {
                        'system-logs': 'emr-managed',
                        'application-logs': 'on-customer-s3only',
                        'persistent-ui-logs': 'disabled',
                    },
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_s3_logging_valid_all_disabled_no_log_uri(self):
        """Test valid S3LoggingConfiguration with all logs disabled and no LogUri"""
        cmd = (
            self.prefix
            + '--release-label emr-7.13.0 '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={system-logs=disabled,application-logs=disabled,persistent-ui-logs=disabled}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-7.13.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'MonitoringConfiguration': {
                'S3LoggingConfiguration': {
                    'LogTypeUploadPolicy': {
                        'system-logs': 'disabled',
                        'application-logs': 'disabled',
                        'persistent-ui-logs': 'disabled',
                    },
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_s3_logging_empty_log_type_upload_policy(self):
        """Test that empty LogTypeUploadPolicy is treated as no S3LoggingConfiguration.

        Empty dict means no S3LoggingConfiguration, so no validation is performed.
        """
        cmd = (
            self.prefix
            + '--release-label emr-5.34.0 '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-5.34.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'MonitoringConfiguration': {
                'S3LoggingConfiguration': {
                    'LogTypeUploadPolicy': {},
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_s3_logging_only_persistent_ui_logs_with_log_uri(self):
        """Test that specifying only persistent-ui-logs with LogUri succeeds.

        Missing log types (system-logs, application-logs) default to
        requiring LogUri, so providing LogUri is valid and required.
        """
        cmd = (
            self.prefix
            + '--release-label emr-5.34.0 '
            + '--log-uri s3://mybucket/logs '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={persistent-ui-logs=emr-managed}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-5.34.0',
            'LogUri': 's3://mybucket/logs',
            'VisibleToAllUsers': True,
            'Tags': [],
            'MonitoringConfiguration': {
                'S3LoggingConfiguration': {
                    'LogTypeUploadPolicy': {
                        'persistent-ui-logs': 'emr-managed',
                    },
                },
            },
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_s3_logging_only_persistent_ui_logs_no_log_uri(self):
        """Test S3LoggingConfiguration with only persistent-ui-logs requires LogUri.

        Missing log types (system-logs, application-logs) default to
        requiring LogUri (they use "default" policy internally, which resolves to "emr-managed").
        """
        cmd = (
            self.prefix
            + '--release-label emr-5.34.0 '
            + '--monitoring-configuration '
            + 'S3LoggingConfiguration={LogTypeUploadPolicy={persistent-ui-logs=emr-managed}} '
            + '--instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        self.assert_error_msg(cmd, 'S3LoggingConfigurationLogUriRequiredError', rc=252)

    def test_create_cluster_with_log_encryption_kms_key_id(self):
        test_log_uri = 's3://test/logs'
        test_log_encryption_kms_key_id = 'valid_kms_key'
        cmd = (
            self.prefix
            + '--release-label emr-5.30.0 '
            + '--log-uri '
            + test_log_uri
            + ' --log-encryption-kms-key-id '
            + test_log_encryption_kms_key_id
            + ' --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-5.30.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'LogUri': test_log_uri,
            'LogEncryptionKmsKeyId': test_log_encryption_kms_key_id,
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_placement_groups(self):
        cmd = (
            self.prefix
            + '--release-label emr-5.28.0 --security-configuration '
            + 'MySecurityConfig --placement-group-configs '
            + 'InstanceRole=MASTER,PlacementStrategy=SPREAD '
            + '--instance-groups '
            + HA_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': HA_INSTANCES,
            'ReleaseLabel': 'emr-5.28.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'PlacementGroupConfigs': [
                {'InstanceRole': 'MASTER', 'PlacementStrategy': 'SPREAD'}
            ],
            'SecurityConfiguration': 'MySecurityConfig',
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_fleets_with_resize_allocation_strategy_spot_od(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-fleets '
            + CONSTANTS_FLEET.INSTANCE_FLEETS_WITH_RESIZE_ALLOCATION_STRATEGY_SPOT_AND_OD
            + ' --ec2-attributes AvailabilityZones=[us-east-1a,us-east-1b]'
        )
        instance_fleets = CONSTANTS_FLEET.RES_INSTANCE_FLEETS_WITH_RESIZE_ALLOCATION_STRATEGY_SPOT_AND_OD
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceFleets': instance_fleets,
                'Placement': {
                    'AvailabilityZones': ['us-east-1a', 'us-east-1b']
                },
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_os_release_label(self):
        test_os_release_label = '2.0.20220406.1'
        cmd = (
            self.prefix
            + '--release-label emr-6.6.0'
            + ' --os-release-label '
            + test_os_release_label
            + ' --instance-groups '
            + DEFAULT_INSTANCE_GROUPS_ARG
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': DEFAULT_INSTANCES,
            'ReleaseLabel': 'emr-6.6.0',
            'VisibleToAllUsers': True,
            'Tags': [],
            'OSReleaseLabel': test_os_release_label,
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_fleets_with_spot_allocation_strategy(self):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-fleets '
            + CONSTANTS_FLEET.INSTANCE_FLEETS_WITH_SPOT_ALLOCATION_STRATEGY
            + ' --ec2-attributes AvailabilityZones=[us-east-1a,us-east-1b]'
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceFleets': CONSTANTS_FLEET.RES_INSTANCE_FLEETS_WITH_SPOT_ALLOCATION_STRATEGY,
                'Placement': {
                    'AvailabilityZones': ['us-east-1a', 'us-east-1b']
                },
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_instance_fleets_with_prioritized_allocation_strategy_spot_ondemand(
        self,
    ):
        cmd = (
            self.prefix
            + '--release-label emr-4.2.0 --instance-fleets '
            + CONSTANTS_FLEET.INSTANCE_FLEETS_WITH_PRIORITIZED_ALLOCATION_STRATEGY_SPOT_AND_OD
            + ' --ec2-attributes AvailabilityZones=[us-east-1a,us-east-1b]'
        )
        result = {
            'Name': DEFAULT_CLUSTER_NAME,
            'Instances': {
                'KeepJobFlowAliveWhenNoSteps': True,
                'TerminationProtected': False,
                'InstanceFleets': CONSTANTS_FLEET.RES_INSTANCE_FLEETS_WITH_PRIORITIZED_ALLOCATION_STRATEGY_SPOT_AND_OD,
                'Placement': {
                    'AvailabilityZones': ['us-east-1a', 'us-east-1b']
                },
            },
            'ReleaseLabel': 'emr-4.2.0',
            'VisibleToAllUsers': True,
            'Tags': [],
        }
        self.assert_params_for_cmd(cmd, result)

    def test_create_cluster_with_steps_with_step_monitoring_configuration(
        self,
    ):
        # For ease of testing, we only test the handling of StepMonitoringConfig in create-cluster
        # with a CUSTOM_JAR step. Under the hood, it uses the same handler as add-steps. The remaining
        # step types are unit tested in test_add_steps.py.
        cmd = (
            DEFAULT_CMD
            + '--steps '
            + (
                'Name=Custom,Type=Custom_JAR,'
                'Jar=s3://mybucket/mytest.jar,'
                'Args=arg1,arg2,MainClass=mymainclass,'
                'ActionOnFailure=TERMINATE_CLUSTER,'
                'LogUri=TestLogUri,'
                'EncryptionKeyArn=TestEncryptionKeyArn'
            )
        )
        expected_steps = [
            {
                'Name': 'Custom',
                'ActionOnFailure': 'TERMINATE_CLUSTER',
                'HadoopJarStep': {
                    'Jar': 's3://mybucket/mytest.jar',
                    'Args': ['arg1', 'arg2'],
                    'MainClass': 'mymainclass',
                },
                'StepMonitoringConfiguration': {
                    'S3MonitoringConfiguration': {
                        'LogUri': 'TestLogUri',
                        'EncryptionKeyArn': 'TestEncryptionKeyArn',
                    }
                },
            }
        ]
        result = copy.deepcopy(DEFAULT_RESULT)
        result['Steps'] = expected_steps
        self.assert_params_for_cmd(cmd, result)


if __name__ == "__main__":
    unittest.main()
